@@ -17,14 +17,12 @@
  * along with this program; if not, write to the Free Software Foundation,
  * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  * 
- * @(#) $Id: disman-script-mib-proc.h,v 1.2 2001/09/30 20:33:47 schoenw Exp $
+ * @(#) $Id: disman-script-mib-proc.h 709 2004-10-14 20:11:17Z schoenw $
  */
 
 #ifndef _DISMAN_SCRIPT_MIB_PROC_H_
 #define _DISMAN_SCRIPT_MIB_PROC_H_
 
-#include "g_snmp.h"
-
 #include "disman-script-mib.h"
 
 /*
@@ -35,13 +33,13 @@
  */
 
 extern void
-disman_script_mib_proc_create_script(GSnmpSession *s,
+disman_script_mib_proc_create_script(GNetSnmp *s,
 				     guchar *owner,
 				     guchar *name,
 				     guchar *descr);
 
-void
-disman_script_mib_proc_create_run(GSnmpSession *s,
+extern void
+disman_script_mib_proc_create_run(GNetSnmp *s,
 				  guchar *lowner,
 				  guchar *lname,
 				  guchar *args);
