@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software Foundation,
  * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  * 
- * @(#) $Id: if-mib-proc.c,v 1.6 2002/07/19 09:11:03 schoenw Exp $
+ * @(#) $Id: if-mib-proc.c 709 2004-10-14 20:11:17Z schoenw $
  */
 
 #include "if-mib-proc.h"
@@ -34,7 +34,7 @@ typedef struct {
 
 
 void
-if_mib_proc_get_ifTable(GSnmpSession *s,
+if_mib_proc_get_ifTable(GNetSnmp *s,
 			if_mib_ifEntry_t ***ifEntry,
 			gint mask,
 			time_t epoch)
@@ -51,7 +51,7 @@ if_mib_proc_get_ifTable(GSnmpSession *s,
 	cache.data = NULL;
     }
 
-    if_mib_get_ifTable(s, ifEntry, mask);
+    if_mib_get_ifTable(s, ifEntry, 0);
     if (! s->error_status) {
 	cache.data = *ifEntry;
 	cache.time = now;
