@@ -17,14 +17,12 @@
  * along with this program; if not, write to the Free Software Foundation,
  * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  * 
- * @(#) $Id: if-mib-proc.h,v 1.5 2002/07/19 09:11:03 schoenw Exp $
+ * @(#) $Id: if-mib-proc.h 709 2004-10-14 20:11:17Z schoenw $
  */
 
 #ifndef _IF_MIB_PROC_H_
 #define _IF_MIB_PROC_H_
 
-#include "g_snmp.h"
-
 #include "if-mib.h"
 
 /*
@@ -35,7 +33,7 @@
  */
 
 extern void
-if_mib_proc_get_ifTable(GSnmpSession *s,
+if_mib_proc_get_ifTable(GNetSnmp *s,
 			if_mib_ifEntry_t ***ifEntry,
 			gint mask,
 			time_t epoch);
