@@ -17,25 +17,22 @@
  * along with this program; if not, write to the Free Software Foundation,
  * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  * 
- * @(#) $Id: productmib-proc.h,v 1.4 2002/06/25 14:31:50 schoenw Exp $
+ * @(#) $Id: productmib-proc.h 709 2004-10-14 20:11:17Z schoenw $
  */
 
 #ifndef _PRODUCTMIB_PROC_H_
 #define _PRODUCTMIB_PROC_H_
 
-#include "g_snmp.h"
-
 #include "productmib.h"
 
-
 extern void
-productmib_proc_create_vlan(GSnmpSession *s, gint32 vlanid,
+productmib_proc_create_vlan(GNetSnmp *s, gint32 vlanid,
 			    guchar *name, gsize name_len, guint32 type);
 extern void
-productmib_proc_delete_vlan(GSnmpSession *s, gint32 ifIndex);
+productmib_proc_delete_vlan(GNetSnmp *s, gint32 ifIndex);
 
 extern void
-productmib_proc_set_vlan_port_member(GSnmpSession *s, gint32 ifIndex,
+productmib_proc_set_vlan_port_member(GNetSnmp *s, gint32 ifIndex,
 				     guchar *ports, gsize ports_len);
     
 #endif /* _PRODUCTMIB_PROC_H_ */
