@@ -17,21 +17,22 @@
  * along with this program; if not, write to the Free Software Foundation,
  * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  * 
- * @(#) $Id: rapid-city-proc.c,v 1.2 2002/06/25 14:31:51 schoenw Exp $
+ * @(#) $Id: rapid-city-proc.c 709 2004-10-14 20:11:17Z schoenw $
  */
 
+#include "snmpv2-tc.h"
+
 #include "rapid-city-proc.h"
 
-
 void
-rapid_city_proc_create_vlan(GSnmpSession *s,
+rapid_city_proc_create_vlan(GNetSnmp *s,
 			    gint32 vlanid,
 			    guchar *name,
 			    gsize name_len,
 			    guint32 type)
 {
     rapid_city_rcVlanEntry_t *vlanEntry;
-    gint32 createAndGo = RAPID_CITY_RCVLANROWSTATUS_CREATEANDGO;
+    gint32 createAndGo = SNMPV2_TC_ROWSTATUS_CREATEANDGO;
     
     vlanEntry = rapid_city_new_rcVlanEntry();
     if (vlanEntry) {
@@ -51,10 +52,10 @@ rapid_city_proc_create_vlan(GSnmpSession *s,
 
 
 void
-rapid_city_proc_delete_vlan(GSnmpSession *s, gint32 vlanid)
+rapid_city_proc_delete_vlan(GNetSnmp *s, gint32 vlanid)
 {
     rapid_city_rcVlanEntry_t *vlanEntry;
-    gint32 destroy = RAPID_CITY_RCVLANROWSTATUS_DESTROY;
+    gint32 destroy = SNMPV2_TC_ROWSTATUS_DESTROY;
 
     rapid_city_get_rcVlanEntry(s, &vlanEntry, vlanid,
 			       RAPID_CITY_RCVLANROWSTATUS);
@@ -68,7 +69,7 @@ rapid_city_proc_delete_vlan(GSnmpSession *s, gint32 vlanid)
 
 
 void
-rapid_city_proc_set_vlan_port_default(GSnmpSession *s,
+rapid_city_proc_set_vlan_port_default(GNetSnmp *s,
 				      gint32 port,
 				      gint32 vlanid)
 {
@@ -86,7 +87,7 @@ rapid_city_proc_set_vlan_port_default(GSnmpSession *s,
 
 
 void
-rapid_city_proc_set_vlan_port_member(GSnmpSession *s,
+rapid_city_proc_set_vlan_port_member(GNetSnmp *s,
 				     gint32 vlanid,
 				     guchar *ports)
 {
