@@ -17,28 +17,25 @@
  * along with this program; if not, write to the Free Software Foundation,
  * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  * 
- * @(#) $Id: rapid-city-proc.h,v 1.2 2002/06/25 14:31:51 schoenw Exp $
+ * @(#) $Id: rapid-city-proc.h 709 2004-10-14 20:11:17Z schoenw $
  */
 
 #ifndef _RAPID_CITY_PROC_H_
 #define _RAPID_CITY_PROC_H_
 
-#include "g_snmp.h"
-
 #include "rapid-city.h"
 
-
 extern void
-rapid_city_proc_create_vlan(GSnmpSession *s, gint32 vlanid,
+rapid_city_proc_create_vlan(GNetSnmp *s, gint32 vlanid,
 			    guchar *name, gsize name_len, guint32 type);
 extern void
-rapid_city_proc_delete_vlan(GSnmpSession *s, gint32 vlanid);
+rapid_city_proc_delete_vlan(GNetSnmp *s, gint32 vlanid);
 
 extern void
-rapid_city_proc_set_vlan_port_default(GSnmpSession *s, gint32 port,
+rapid_city_proc_set_vlan_port_default(GNetSnmp *s, gint32 port,
 				      gint32 vlanid);
 extern void
-rapid_city_proc_set_vlan_port_member(GSnmpSession *s, gint32 vlanid,
+rapid_city_proc_set_vlan_port_member(GNetSnmp *s, gint32 vlanid,
 				     guchar *ports);
     
 #endif /* _RAPID_CITY_PROC_H_ */
