@@ -17,24 +17,22 @@
  * along with this program; if not, write to the Free Software Foundation,
  * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  * 
- * @(#) $Id: snmp-user-based-sm-mib-proc.h,v 1.1 2002/06/18 09:29:25 schoenw Exp $
+ * @(#) $Id: snmp-user-based-sm-mib-proc.h 709 2004-10-14 20:11:17Z schoenw $
  */
 
 #ifndef _SNMP_USER_BASED_SM_MIB_PROC_H_
 #define _SNMP_USER_BASED_SM_MIB_PROC_H_
 
-#include "g_snmp.h"
-
 #include "snmp-framework-mib.h"
 #include "snmp-user-based-sm-mib.h"
 
 extern void
-snmp_user_based_sm_mib_proc_clone_user(GSnmpSession *s,
+snmp_user_based_sm_mib_proc_clone_user(GNetSnmp *s,
 				       guchar *user, size_t user_len,
 				       guchar *dolly, size_t dolly_len);
 
 extern void
-snmp_user_based_sm_mib_proc_delete_user(GSnmpSession *s,
+snmp_user_based_sm_mib_proc_delete_user(GNetSnmp *s,
 					guchar *name, gsize name_len,
 					guchar *group, gsize group_len,
 					gint32 model);
