@@ -17,23 +17,25 @@
  * along with this program; if not, write to the Free Software Foundation,
  * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  * 
- * @(#) $Id: snmp-view-based-acm-mib-proc.c,v 1.4 2002/07/19 09:11:03 schoenw Exp $
+ * @(#) $Id: snmp-view-based-acm-mib-proc.c 1995 2006-08-22 21:58:46Z schoenw $
  */
 
+#include "snmpv2-tc.h"
+
 #include "snmp-view-based-acm-mib-proc.h"
 
 void
-snmp_view_based_acm_mib_proc_create_member(GSnmpSession *s,
+snmp_view_based_acm_mib_proc_create_member(GNetSnmp *s,
 					   guchar *name,
 					   guchar *group,
 					   gint32 model)
 {
     snmp_view_based_acm_mib_vacmSecurityToGroupEntry_t *vacmGroupEntry;
-    gint32 createAndGo = SNMP_VIEW_BASED_ACM_MIB_VACMSECURITYTOGROUPSTATUS_CREATEANDGO;
+    gint32 createAndGo = SNMPV2_TC_ROWSTATUS_CREATEANDGO;
 
     vacmGroupEntry = snmp_view_based_acm_mib_new_vacmSecurityToGroupEntry();
     if (! vacmGroupEntry) {
-	s->error_status = G_SNMP_ERR_PROCEDURE;
+	s->error_status = GNET_SNMP_PDU_ERR_PROCEDURE;
 	return;
     }
     vacmGroupEntry->_vacmSecurityNameLength = strlen(name);
@@ -48,17 +50,17 @@ snmp_view_based_acm_mib_proc_create_member(GSnmpSession *s,
 }
 
 void
-snmp_view_based_acm_mib_proc_delete_member(GSnmpSession *s,
+snmp_view_based_acm_mib_proc_delete_member(GNetSnmp *s,
 					   guchar *name, gsize name_len,
 					   guchar *group, gsize group_len,
 					   gint32 model)
 {
     snmp_view_based_acm_mib_vacmSecurityToGroupEntry_t *vacmGroupEntry;
-    gint32 destroy = SNMP_VIEW_BASED_ACM_MIB_VACMSECURITYTOGROUPSTATUS_DESTROY;
+    gint32 destroy = SNMPV2_TC_ROWSTATUS_DESTROY;
 
     vacmGroupEntry = snmp_view_based_acm_mib_new_vacmSecurityToGroupEntry();
     if (! vacmGroupEntry) {
-	s->error_status = G_SNMP_ERR_PROCEDURE;
+	s->error_status = GNET_SNMP_PDU_ERR_PROCEDURE;
 	return;
     }
     vacmGroupEntry->_vacmSecurityNameLength = name_len;
@@ -74,7 +76,7 @@ snmp_view_based_acm_mib_proc_delete_member(GSnmpSession *s,
 
 #if 0
 void
-disman_script_mib_proc_create_script(GSnmpSession *s,
+disman_script_mib_proc_create_script(GNetSnmp *s,
 				     guchar *owner,
 				     guchar *name,
 				     guchar *descr)
