@@ -17,24 +17,22 @@
  * along with this program; if not, write to the Free Software Foundation,
  * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  * 
- * @(#) $Id: snmp-view-based-acm-mib-proc.h,v 1.3 2002/02/15 14:08:32 schoenw Exp $
+ * @(#) $Id: snmp-view-based-acm-mib-proc.h 709 2004-10-14 20:11:17Z schoenw $
  */
 
 #ifndef _SNMP_VIEW_BASED_ACM_MIB_PROC_H_
 #define _SNMP_VIEW_BASED_ACM_MIB_PROC_H_
 
-#include "g_snmp.h"
-
 #include "snmp-view-based-acm-mib.h"
 
 extern void
-snmp_view_based_acm_mib_proc_create_member(GSnmpSession *s,
+snmp_view_based_acm_mib_proc_create_member(GNetSnmp *s,
 					   guchar *name,
 					   guchar *group,
 					   gint32 model);
 
 extern void
-snmp_view_based_acm_mib_proc_delete_member(GSnmpSession *s,
+snmp_view_based_acm_mib_proc_delete_member(GNetSnmp *s,
 					   guchar *name, gsize name_len,
 					   guchar *group, gsize group_len,
 					   gint32 model);
