@@ -17,14 +17,12 @@
  * along with this program; if not, write to the Free Software Foundation,
  * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  * 
- * @(#) $Id: udp-mib-proc.h,v 1.1 2002/02/07 10:14:59 schoenw Exp $
+ * @(#) $Id: udp-mib-proc.h 709 2004-10-14 20:11:17Z schoenw $
  */
 
 #ifndef _UDP_MIB_PROC_H_
 #define _UDP_MIB_PROC_H_
 
-#include "g_snmp.h"
-
 #include "udp-mib.h"
 
 /*
@@ -35,14 +33,21 @@
  */
 
 typedef struct {
-    guint32 inDatagrams;
-    guint32 outDatagrams;
-    guint32 inDiscards;
+    guint32 udpInDatagrams;
+    guint32 udpOutDatagrams;
+    guint32 udpInDiscards;
+    guint32 _udpInDatagrams;
+    guint32 _udpNoPorts;
+    guint32 _udpInErrors;
+    guint32 _udpOutDatagrams;
+    guint32 _sysUpTime;
+    /* handle to detect invalidated sessions ? */
+    /* handle to detect discontinuities ? */
 } udp_mib_proc_stats_t;
 
 
 extern void
-udp_mib_proc_get_stats(GSnmpSession *s,
+udp_mib_proc_get_stats(GNetSnmp *s,
 		       udp_mib_proc_stats_t **udpStatsEntry);
 
 
