@@ -17,12 +17,14 @@
  * along with this program; if not, write to the Free Software Foundation,
  * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  * 
- * @(#) $Id: entity.c,v 1.32 2002/07/24 06:54:30 schoenw Exp $
+ * @(#) $Id: entity.c 1758 2006-01-18 09:25:44Z schoenw $
  */
 
 #include "scli.h"
 
+#include "snmpv2-tc.h"
 #include "entity-mib.h"
+#include "entity-sensor-mib.h"
 
 
 static void
@@ -55,10 +57,10 @@ fmt_containment(GString *s, char *prefix,
 		}
 	    }
 	    
-	    g_string_sprintfa(s, "%6d",
+	    g_string_sprintfa(s, "%10d",
 			      entPhysicalEntry[i]->entPhysicalIndex);
 
-	    class = fmt_enum(entity_mib_enums_entPhysicalClass,
+	    class = fmt_enum(entity_mib_enums_PhysicalClass,
 			     entPhysicalEntry[i]->entPhysicalClass);
 	    g_string_sprintfa(s, " %-*s", class_width, class ? class : "");
 
@@ -116,13 +118,13 @@ show_entity_containment(scli_interp_t *interp, int argc, char **argv)
 
     if (entPhysicalTable) {
 	for (i = 0; entPhysicalTable[i]; i++) {
-	    class = fmt_enum(entity_mib_enums_entPhysicalClass,
+	    class = fmt_enum(entity_mib_enums_PhysicalClass,
 			     entPhysicalTable[i]->entPhysicalClass);
 	    if (strlen(class) > class_width) {
 		class_width = strlen(class);
 	    }
 	}
-	g_string_sprintfa(interp->header, "ENTITY %-*s CONTAINMENT",
+	g_string_sprintfa(interp->header, "    ENTITY %-*s CONTAINMENT",
 			  class_width, "CLASS");
 	fmt_containment(interp->result, "", entPhysicalTable, 0, class_width);
     }
@@ -134,6 +136,50 @@ show_entity_containment(scli_interp_t *interp, int argc, char **argv)
 
 
 
+static void
+fmt_entity_sensors(GString *s,
+		   entity_sensor_mib_entPhySensorEntry_t *entSensorEntry)
+{
+    g_string_sprintfa(s, "sensor %d\n",
+		      entSensorEntry->entPhysicalIndex);
+}
+
+
+
+static int
+show_entity_sensors(scli_interp_t *interp, int argc, char **argv)
+{
+    entity_sensor_mib_entPhySensorEntry_t **entSensorTable = NULL;
+    int i;
+
+    g_return_val_if_fail(interp, SCLI_ERROR);
+
+    if (argc > 1) {
+	return SCLI_SYNTAX_NUMARGS;
+    }
+
+    if (scli_interp_dry(interp)) {
+	return SCLI_OK;
+    }
+
+    entity_sensor_mib_get_entPhySensorTable(interp->peer, &entSensorTable, 0);
+    if (interp->peer->error_status) {
+	return SCLI_SNMP;
+    }
+
+    if (entSensorTable) {
+	for (i = 0; entSensorTable[i]; i++) {
+	    fmt_entity_sensors(interp->result, entSensorTable[i]);
+	}
+    }
+
+    if (entSensorTable) entity_sensor_mib_free_entPhySensorTable(entSensorTable);
+    
+    return SCLI_OK;
+}
+
+
+
 static void
 fmt_entity_details(GString *s, entity_mib_entPhysicalEntry_t *entPhysicalEntry)
 {
@@ -203,12 +249,13 @@ fmt_entity_details(GString *s, entity_mib_entPhysicalEntry_t *entPhysicalEntry)
 	g_string_sprintfa(s, "Asset:  %-30s", "");
     }
 
-    class = fmt_enum(entity_mib_enums_entPhysicalClass,
+    class = fmt_enum(entity_mib_enums_PhysicalClass,
 		     entPhysicalEntry->entPhysicalClass);
     if (class) {
 	g_string_sprintfa(s, "Class:  %s", class);
 	if (entPhysicalEntry->entPhysicalIsFRU) {
-	    if (*entPhysicalEntry->entPhysicalIsFRU == 2) {
+	    if (*entPhysicalEntry->entPhysicalIsFRU
+		== SNMPV2_TC_TRUTHVALUE_TRUE) {
 		g_string_append(s, " (replaceable)");
 	    }
 	}
@@ -247,7 +294,7 @@ xml_entity_details(xmlNodePtr root,
 			     entPhysicalEntry->entPhysicalName);
     }
 
-    e = fmt_enum(entity_mib_enums_entPhysicalClass,
+    e = fmt_enum(entity_mib_enums_PhysicalClass,
 		 entPhysicalEntry->entPhysicalClass);
     if (e) {
 	(void) xml_new_child(tree, NULL, "class", "%s", e);
@@ -260,7 +307,7 @@ xml_entity_details(xmlNodePtr root,
 
     if (entPhysicalEntry->entPhysicalIsFRU
 	&& (*entPhysicalEntry->entPhysicalIsFRU
-	    == ENTITY_MIB_ENTPHYSICALISFRU_TRUE)) {
+	    == SNMPV2_TC_TRUTHVALUE_TRUE)) {
 	(void) xmlNewChild(tree, NULL, "replaceable", NULL);
     } 
  
@@ -361,9 +408,9 @@ fmt_entity_info(GString *s, entity_mib_entPhysicalEntry_t *entPhysicalEntry,
 {
     const char *class;
 
-    g_string_sprintfa(s, "%6d", entPhysicalEntry->entPhysicalIndex);
+    g_string_sprintfa(s, "%10d", entPhysicalEntry->entPhysicalIndex);
     
-    class = fmt_enum(entity_mib_enums_entPhysicalClass,
+    class = fmt_enum(entity_mib_enums_PhysicalClass,
 		     entPhysicalEntry->entPhysicalClass);
     if (class) {
 	g_string_sprintfa(s, " %-*s", class_width, class);
@@ -414,7 +461,7 @@ show_entity_info(scli_interp_t *interp, int argc, char **argv)
 
     if (entPhysicalTable) {
 	for (i = 0; entPhysicalTable[i]; i++) {
-	    class = fmt_enum(entity_mib_enums_entPhysicalClass,
+	    class = fmt_enum(entity_mib_enums_PhysicalClass,
 			     entPhysicalTable[i]->entPhysicalClass);
 	    if (strlen(class) > class_width) {
 		class_width = strlen(class);
@@ -425,7 +472,7 @@ show_entity_info(scli_interp_t *interp, int argc, char **argv)
 	    }
 	}
 	g_string_sprintfa(interp->header,
-			  "ENTITY %-*s %-*s DESCRIPTION",
+			  "    ENTITY %-*s %-*s DESCRIPTION",
 			  class_width, "CLASS",
 			  name_width, "NAME");
 	for (i = 0; entPhysicalTable[i]; i++) {
@@ -473,6 +520,13 @@ scli_init_entity_mode(scli_interp_t *interp)
 	  NULL, NULL,
 	  show_entity_containment },
 
+	{ "show entity sensors ", NULL,
+	  "The `show entity sensors' command describes the physical sensor\n"
+	  "entities in more detail.",
+	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_XML | SCLI_CMD_FLAG_DRY,
+	  "entities sensors", NULL,
+	  show_entity_sensors },
+	
 #if 0
 	{ "set entity serial", NULL,
 	  "xxx",
@@ -490,7 +544,7 @@ scli_init_entity_mode(scli_interp_t *interp)
 	  "xxx",
 	  SCLI_CMD_FLAG_NEED_PEER,
 	  NULL, NULL,
-	  set_entity_serial },
+	  set_entity_asset },
 
 	{ "dump entity", NULL,
 	  "xxx",
@@ -498,6 +552,11 @@ scli_init_entity_mode(scli_interp_t *interp)
 	  NULL, NULL,
 	  set_entity_serial },
 #endif
+
+#if 0
+	/* ENTITY NAME STATUS UNIT VALUE UPDATE-RATE LAST-UPDATE */
+#endif
+	
 	{ NULL, NULL, NULL, 0, NULL, NULL, NULL }
     };
     
