@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software Foundation,
  * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  * 
- * @(#) $Id: fmt.c,v 1.36 2002/06/25 07:44:19 schoenw Exp $
+ * @(#) $Id: fmt.c 734 2004-11-05 09:23:17Z schoenw $
  */
 
 #include "scli.h"
@@ -52,7 +52,7 @@ static guint32 const snmpIPXDomain[]
 static guint32 const rfc1157Domain[]
 	= { SNMPV2_TM_RFC1157DOMAIN };
 
-static GSnmpIdentity const tdomain_identities[] = {
+static GNetSnmpIdentity const tdomain_identities[] = {
     { snmpUDPDomain,
       sizeof(snmpUDPDomain)/sizeof(guint32),
       "udp" },
@@ -76,8 +76,8 @@ static GSnmpIdentity const tdomain_identities[] = {
 
 
 
-char const *
-fmt_timeticks(guint32 timeticks)
+static char const *
+format_timeticks(guint32 timeticks, int xml)
 {
     static char buffer[80];
     time_t now, gmt;
@@ -85,7 +85,7 @@ fmt_timeticks(guint32 timeticks)
     int gmt_offset;
     
     now = time(NULL);
-    now -= timeticks/100;
+    now -= (timeticks+50)/100;
 
     /*
      * Get the UTC time (which is basically identically with GMT)
@@ -103,6 +103,15 @@ fmt_timeticks(guint32 timeticks)
     tm->tm_isdst = 0;
     gmt_offset = mktime(tm) - gmt;
 
+    if (xml) {
+	g_snprintf(buffer, sizeof(buffer),
+		   "%04d-%02d-%02dT%02d:%02d:%02d%c%02d:%02d",
+		   now_tm.tm_year + 1900, now_tm.tm_mon + 1, now_tm.tm_mday,
+		   now_tm.tm_hour, now_tm.tm_min, now_tm.tm_sec,
+		   gmt_offset >= 0 ? '+' : '-',
+		   (int) ABS(gmt_offset) / 3600,
+		   (int) (ABS(gmt_offset) / 60) % 60);
+    } else {
     g_snprintf(buffer, sizeof(buffer),
 	       "%04d-%02d-%02d %02d:%02d:%02d %c%02d:%02d",
 	       now_tm.tm_year + 1900, now_tm.tm_mon + 1, now_tm.tm_mday,
@@ -110,12 +119,29 @@ fmt_timeticks(guint32 timeticks)
 	       gmt_offset >= 0 ? '+' : '-',
 	       (int) ABS(gmt_offset) / 3600,
 	       (int) (ABS(gmt_offset) / 60) % 60);
+    }
     
     return buffer;
 }
 
 
 
+char const *
+xml_timeticks(guint32 timeticks)
+{
+    return format_timeticks(timeticks, 1);
+}
+
+
+
+char const *
+fmt_timeticks(guint32 timeticks)
+{
+    return format_timeticks(timeticks, 0);
+}
+
+
+
 static time_t
 date_to_time(guchar *date, gsize len)
 {
@@ -138,8 +164,8 @@ date_to_time(guchar *date, gsize len)
 
 
 
-char const *
-fmt_date_and_time(guchar *data, gsize len)
+static char const *
+format_date_and_time(guchar *data, gsize len, int xml)
 {
     static char buffer[80];
     int i;
@@ -169,22 +195,53 @@ fmt_date_and_time(guchar *data, gsize len)
      */
     
     if (len == 8) {
+	if (xml) {
+	    g_snprintf(buffer, sizeof(buffer),
+		       "%04u-%02u-%02uT%02u:%02u:%02u",
+		       (data[0] << 8) + data[1], data[2], data[3],
+		       data[4], data[5], data[6]);
+	} else {
 	g_snprintf(buffer, sizeof(buffer),
 		   "%04u-%02u-%02u %02u:%02u:%02u",
 		   (data[0] << 8) + data[1], data[2], data[3],
 		   data[4], data[5], data[6]);
+	}
     } else if (len == 11) {
+	if (xml) {
+	    g_snprintf(buffer, sizeof(buffer),
+		       "%04u-%02u-%02uT%02u:%02u:%02u%c%02u:%02u",
+		       (data[0] << 8) + data[1], data[2], data[3],
+		       data[4], data[5], data[6],
+		       data[8], data[9], data[10]);
+	} else {
 	g_snprintf(buffer, sizeof(buffer),
 		   "%04u-%02u-%02u %02u:%02u:%02u %c%02u:%02u",
 		   (data[0] << 8) + data[1], data[2], data[3],
 		   data[4], data[5], data[6],
 		   data[8], data[9], data[10]);
     }
+    }
     return buffer;
 }
 
 
 
+char const *
+xml_date_and_time(guchar *data, gsize len)
+{
+    return format_date_and_time(data, len, 1);
+}
+
+
+
+char const *
+fmt_date_and_time(guchar *data, gsize len)
+{
+    return format_date_and_time(data, len, 0);
+}
+
+
+
 char const *
 fmt_date_and_time_delta(guchar *date1, gsize len1,
 			guchar *date2, gsize len2)
@@ -209,15 +266,35 @@ fmt_seconds(guint32 seconds)
     min  = (seconds / 60) % 60;
     hour = (seconds / 60 / 60);
 
-    g_snprintf(buffer, sizeof(buffer), "%3d:%02d:%02d", hour, min, sec);
+    g_snprintf(buffer, sizeof(buffer), "%4d:%02d:%02d", hour, min, sec);
 
     return buffer;
 }
 
 
 
+char const *
+fmt_enum(GNetSnmpEnum const *table, gint32 *number)
+{
+    static char buffer[80];
+    gchar const *name;
+
+    if (! number) {
+	return NULL;
+    }
+
+    name = gnet_snmp_enum_get_label(table, *number);
+    if (! name) {
+	g_snprintf(buffer, sizeof(buffer), "%d", *number);
+	name = buffer;
+    }
+    return name;
+}
+
+
+
 void
-xxx_enum(GString *s, int width, GSnmpEnum const *table, gint32 *number)
+xxx_enum(GString *s, int width, GNetSnmpEnum const *table, gint32 *number)
 {
     gchar const *name;
 
@@ -232,22 +309,25 @@ xxx_enum(GString *s, int width, GSnmpEnum const *table, gint32 *number)
 
 
 char const *
-fmt_enum(GSnmpEnum const *table, gint32 *number)
+fmt_identity(GNetSnmpIdentity const *table,
+	     guint32 const *oid, gsize oidlen)
 {
+#if 0
     static char buffer[80];
+#endif
     gchar const *name;
 
-    if (! number) {
+    if (! oid) {
 	return NULL;
     }
 
-    name = gsnmp_enum_get_label(table, *number);
+    name = gnet_snmp_identity_get_label(table, oid, oidlen);
+#if 0
     if (! name) {
-	g_snprintf(buffer, sizeof(buffer), "%d", *number);
-	return buffer;
-    } else {
-	return name;
+	/* xxx */
     }
+#endif
+    return name;
 }
 
 
@@ -438,7 +518,7 @@ fmt_indent_string(GString *s, int indent, char *label, int len, char *string)
 	if (string[i] == '\n') {
 	    g_string_sprintfa(s, "\n%*s", indent, "");
 	} else {
-	    if (isprint(string[i])) {
+	    if (isprint((int) string[i])) {
 		g_string_append_c(s, string[i]);
 	    } else {
 		g_string_sprintfa(s, "\\%o", string[i]);
@@ -449,6 +529,34 @@ fmt_indent_string(GString *s, int indent, char *label, int len, char *string)
 }
 
 
+
+const char *
+xml_display_string(int len, char *string)
+{
+    static char *e = NULL;
+    
+    if (e) {
+	g_free(e);
+	e = NULL;
+    }
+
+    /* Remove leading and trailing white-space characters first. */
+
+    while (len && isspace((int) string[0])) {
+	string++, len--;
+    }
+    while (len && isspace((int) string[len-1])) {
+	len--;
+    }
+
+    e = g_malloc0(len+1);
+    snprintf(e, len+1, "%s\0", string);
+    
+    return e;
+}
+
+
+
 void
 fmt_display_string(GString *s, int indent, char *label, int len, char *string)
 {
@@ -459,10 +567,10 @@ fmt_display_string(GString *s, int indent, char *label, int len, char *string)
 
     /* Remove leading and trailing white-space characters first. */
 
-    while (len && isspace(string[0])) {
+    while (len && isspace((int) string[0])) {
 	string++, len--;
     }
-    while (len && isspace(string[len-1])) {
+    while (len && isspace((int) string[len-1])) {
 	len--;
     }
 
@@ -472,19 +580,19 @@ fmt_display_string(GString *s, int indent, char *label, int len, char *string)
     
     for (i = 0; i < len; i++) {
 	if (string[i] == '\r') continue;
-	if (isspace(string[i])) {
+	if (isspace((int) string[i])) {
 	    int j, p;
-	    for (j = i+1, p = pos; p < 80 && j < len; p++, j++) {
-		if (string[j] != '\r' && isspace(string[j])) break;
+	    for (j = i+1, p = pos; p < 76 && j < len; p++, j++) {
+		if (string[j] != '\r' && isspace((int) string[j])) break;
 	    }
-	    if (p == 80) {
+	    if (p == 76) {
 		g_string_sprintfa(s, "\n%*s", indent, "");
 		pos = indent;
 		continue;
 	    }
 	}
 	if (string[i] != '\n') {
-	    if (isprint(string[i])) {
+	    if (isprint((int) string[i])) {
 		g_string_append_c(s, string[i]);
 		pos++;
 	    } else {
@@ -503,7 +611,7 @@ fmt_tdomain(guint32 *tdomain, gsize tdomain_len)
 {
     const char *e;
 
-    e = gsnmp_identity_get_label(tdomain_identities, tdomain, tdomain_len);
+    e = fmt_identity(tdomain_identities, tdomain, tdomain_len);
 
     return e;
 }
@@ -522,7 +630,7 @@ fmt_taddress(guint32 *tdomain, gsize tdomain_len,
 	e = NULL;
     }
 
-    if (! tdomain && taddress) {
+    if (! tdomain || !taddress) {
 	return NULL;
     }
 
@@ -565,7 +673,7 @@ fmt_taddress(guint32 *tdomain, gsize tdomain_len,
 void
 fmt_row_status(GString *s, gint32 *status)
 {
-    static GSnmpEnum const row_states[] = {
+    static GNetSnmpEnum const row_states[] = {
 	{ 1, "A" },	/* active */
 	{ 2, "S" },	/* notInService */
 	{ 3, "R" },	/* notReady */
@@ -579,7 +687,7 @@ fmt_row_status(GString *s, gint32 *status)
 	return;
     }
 
-    label = gsnmp_enum_get_label(row_states, *status);
+    label = gnet_snmp_enum_get_label(row_states, *status);
     if (label) {
 	g_string_append(s, label);
     } else {
@@ -592,7 +700,7 @@ fmt_row_status(GString *s, gint32 *status)
 void
 fmt_storage_type(GString *s, gint32 *storage)
 {
-    static GSnmpEnum const storage_types[] = {
+    static GNetSnmpEnum const storage_types[] = {
 	{ 1, "O" },	/* other */
 	{ 2, "V" },	/* volatile */
 	{ 3, "N" },	/* nonVolatile */
@@ -608,7 +716,7 @@ fmt_storage_type(GString *s, gint32 *storage)
 	return;
     }
     
-    label = gsnmp_enum_get_label(storage_types, *storage);
+    label = gnet_snmp_enum_get_label(storage_types, *storage);
     if (label) {
 	g_string_append(s, label);
     } else {
