@@ -17,16 +17,20 @@
  * along with this program; if not, write to the Free Software Foundation,
  * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  * 
- * @(#) $Id: ip.c,v 1.43 2002/09/02 09:21:26 schoenw Exp $
+ * @(#) $Id: ip.c 1996 2006-08-22 22:13:26Z schoenw $
  */
 
 #include "scli.h"
 
 #include "ip-mib.h"
 #include "ip-forward-mib.h"
+#include "ianaiftype-mib.h"
 #include "tunnel-mib.h"
 #include "if-mib.h"
 #include "rfc1213-mib.h"
+#include "ianaiftype-mib.h"
+
+#include "ianaiftype-mib-proc.h"
 
 #define RFC1213_MIB_IPROUTE_MASK \
 	( RFC1213_MIB_IPROUTEDEST | RFC1213_MIB_IPROUTEIFINDEX \
@@ -34,7 +38,7 @@
 	  | RFC1213_MIB_IPROUTEPROTO | RFC1213_MIB_IPROUTEMASK )
 
 
-GSnmpEnum const forwarding[] = {
+GNetSnmpEnum const forwarding[] = {
     { IP_MIB_IPFORWARDING_FORWARDING,	"enabled" },
     { IP_MIB_IPFORWARDING_NOTFORWARDING,	"disabled" },
     { 0, NULL }
@@ -42,6 +46,23 @@ GSnmpEnum const forwarding[] = {
 
 
 
+static int
+get_if_name_width(if_mib_ifXEntry_t **ifXTable)
+{
+    int i, name_width = 4;
+    
+    if (ifXTable) {
+	for (i = 0; ifXTable[i]; i++) {
+	    if (ifXTable[i]->_ifNameLength > name_width) {
+		name_width = ifXTable[i]->_ifNameLength;
+	    }
+	}
+    }
+    return name_width;
+}
+
+
+
 static void
 fmt_ip_forward(GString *s,
 	       ip_forward_mib_ipCidrRouteEntry_t *ipCidrRouteEntry,
@@ -52,7 +73,6 @@ fmt_ip_forward(GString *s,
     int i, pos;
     
     if (ipCidrRouteEntry->ipCidrRouteIfIndex) {
-	g_string_append(s, "\n");
 	g_string_sprintfa(s, "%s/%s%n",
 			  fmt_ipv4_address(ipCidrRouteEntry->ipCidrRouteDest,
 					   SCLI_FMT_ADDR),
@@ -67,15 +87,15 @@ fmt_ip_forward(GString *s,
 
 	label = NULL;
 	if (ipCidrRouteEntry->ipCidrRouteType) {
-	    label = gsnmp_enum_get_label(ip_forward_mib_enums_ipCidrRouteType,
-					 *ipCidrRouteEntry->ipCidrRouteType);
+	    label = gnet_snmp_enum_get_label(ip_forward_mib_enums_ipCidrRouteType,
+					     *ipCidrRouteEntry->ipCidrRouteType);
 	}
 	g_string_sprintfa(s, "%-10s", label ? label : "");
 
 	label = NULL;
 	if (ipCidrRouteEntry->ipCidrRouteProto) {
-	    label = gsnmp_enum_get_label(ip_forward_mib_enums_ipCidrRouteProto,
-					*ipCidrRouteEntry->ipCidrRouteProto);
+	    label = gnet_snmp_enum_get_label(ip_forward_mib_enums_ipCidrRouteProto,
+					     *ipCidrRouteEntry->ipCidrRouteProto);
 	}
 	g_string_sprintfa(s, "%-10s", label ? label : "");
 
@@ -102,6 +122,7 @@ fmt_ip_forward(GString *s,
 		}
 	    }
 	}
+	g_string_append(s, "\n");
     }
 }
 
@@ -134,15 +155,15 @@ fmt_ip_route(GString *s,
 
 	label = NULL;
 	if (ipRouteEntry->ipRouteType) {
-	    label = gsnmp_enum_get_label(rfc1213_mib_enums_ipRouteType,
-					 *ipRouteEntry->ipRouteType);
+	    label = gnet_snmp_enum_get_label(rfc1213_mib_enums_ipRouteType,
+					     *ipRouteEntry->ipRouteType);
 	}
 	g_string_sprintfa(s, "%-10s", label ? label : "");
 
 	label = NULL;
 	if (ipRouteEntry->ipRouteProto) {
-	    label = gsnmp_enum_get_label(rfc1213_mib_enums_ipRouteProto,
-					*ipRouteEntry->ipRouteProto);
+	    label = gnet_snmp_enum_get_label(rfc1213_mib_enums_ipRouteProto,
+					     *ipRouteEntry->ipRouteProto);
 	}
 	g_string_sprintfa(s, "%-10s", label ? label : "");
 
@@ -243,6 +264,7 @@ xml_ip_address(xmlNodePtr root, ip_mib_ipAddrEntry_t *ipAddrEntry)
     tree = xmlNewChild(root, NULL, "address", NULL);
     xmlSetProp(tree, "address",
 	       fmt_ipv4_address(ipAddrEntry->ipAdEntAddr, SCLI_FMT_ADDR));
+    xmlSetProp(tree, "type", "ipv4");
 
     if (ipAddrEntry->ipAdEntNetMask) {
 	xmlSetProp(tree, "prefix", fmt_ipv4_mask(ipAddrEntry->ipAdEntNetMask));
@@ -265,7 +287,8 @@ fmt_ip_address(GString *s,
 	       ip_mib_ipAddrEntry_t *ipAddrEntry,
 	       int name_width,
 	       if_mib_ifXEntry_t **ifXTable,
-	       if_mib_ifEntry_t **ifTable)
+	       if_mib_ifEntry_t **ifTable,
+	       int ifName_width)
 {
     char *name;
     int i;
@@ -282,22 +305,24 @@ fmt_ip_address(GString *s,
     g_string_sprintfa(s, "%-*s ", name_width, name ? name : "");
     
     if (ipAddrEntry->ipAdEntIfIndex) {
-	g_string_sprintfa(s, " %u", *(ipAddrEntry->ipAdEntIfIndex));
-	if (ifXTable) {
-	    for (i = 0; ifXTable[i]; i++) {
-		if (ifXTable[i]->ifIndex == *ipAddrEntry->ipAdEntIfIndex
-		    && ifXTable[i]->ifName) {
-		    g_string_sprintfa(s, " (%.*s)",
-				      (int) ifXTable[i]->_ifNameLength,
-				      ifXTable[i]->ifName);
-		    break;
-		}
+	g_string_sprintfa(s, "%9u", *(ipAddrEntry->ipAdEntIfIndex));
+	for (i = 0; ifXTable && ifXTable[i]; i++) {
+	    if (ifXTable[i]->ifIndex == *ipAddrEntry->ipAdEntIfIndex
+		&& ifXTable[i]->ifName) {
+		g_string_sprintfa(s, " %-*.*s", ifName_width,
+				  (int) ifXTable[i]->_ifNameLength,
+				  ifXTable[i]->ifName);
+		break;
 	    }
-	} else if (ifTable) {
+	}
+	if (!ifXTable || !ifXTable[i]) {
+	    g_string_sprintfa(s, " %*s", ifName_width, "");
+	}
+	if (ifTable) {
 	    for (i = 0; ifTable[i]; i++) {
 		if (ifTable[i]->ifIndex == *ipAddrEntry->ipAdEntIfIndex
 		    && ifTable[i]->ifDescr) {
-		    g_string_sprintfa(s, " (%.*s)",
+		    g_string_sprintfa(s, " %.*s",
 				      (int) ifTable[i]->_ifDescrLength,
 				      ifTable[i]->ifDescr);
 		    break;
@@ -316,7 +341,7 @@ show_ip_addresses(scli_interp_t *interp, int argc, char **argv)
     ip_mib_ipAddrEntry_t **ipAddrTable = NULL;
     if_mib_ifEntry_t **ifTable = NULL;
     if_mib_ifXEntry_t **ifXTable = NULL;
-    int i, name_width = 4;
+    int i, name_width = 4, ifName_width;
 
     g_return_val_if_fail(interp, SCLI_ERROR);
 
@@ -336,6 +361,7 @@ show_ip_addresses(scli_interp_t *interp, int argc, char **argv)
     if (ipAddrTable) {
 	if_mib_get_ifTable(interp->peer, &ifTable, IF_MIB_IFDESCR);
 	if_mib_get_ifXTable(interp->peer, &ifXTable, IF_MIB_IFNAME);
+	ifName_width = get_if_name_width(ifXTable);
 	if (! scli_interp_xml(interp)) {
 	    for (i = 0; ipAddrTable[i]; i++) {
 		char *name;
@@ -345,15 +371,15 @@ show_ip_addresses(scli_interp_t *interp, int argc, char **argv)
 		}
 	    }
 	    g_string_sprintfa(interp->header,
-			      "ADDRESS         PREFIX %-*s INTERFACE",
-			      name_width, "NAME");
+		      "ADDRESS         PREFIX %-*s INTERFACE %-*s DESCRIPTION",
+			      name_width, "NAME", ifName_width, "NAME");
 	}
 	for (i = 0; ipAddrTable[i]; i++) {
 	    if (scli_interp_xml(interp)) {
 		xml_ip_address(interp->xml_node, ipAddrTable[i]);
 	    } else {
 		fmt_ip_address(interp->result, ipAddrTable[i],
-			       name_width, ifXTable, ifTable);
+			       name_width, ifXTable, ifTable, ifName_width);
 	    }
 	}
     }
@@ -397,7 +423,7 @@ fmt_ip_tunnel(GString *s,
 	g_string_sprintfa(s, "%-15s  ", "-");
     }
 
-    e = fmt_enum(tunnel_mib_enums_tunnelIfEncapsMethod,
+    e = fmt_enum(ianaiftype_mib_enums_IANAtunnelType,
 		 tunnelIfEntry->tunnelIfEncapsMethod);
     g_string_sprintfa(s, "%-8s", e ? e : "-");
 
@@ -531,12 +557,7 @@ fmt_ip_mapping(GString *s,
 	/* See RFC 2665 section 3.2.6. why the test below is so ugly... */
 	if (ifEntry && ifEntry->ifType
 	    && ipNetToMediaEntry->_ipNetToMediaPhysAddressLength == 6
-	    && (*ifEntry->ifType == IF_MIB_IFTYPE_ETHERNETCSMACD
-		|| *ifEntry->ifType == IF_MIB_IFTYPE_ISO88023CSMACD
-		|| *ifEntry->ifType == IF_MIB_IFTYPE_STARLAN
-		|| *ifEntry->ifType == IF_MIB_IFTYPE_FASTETHER
-		|| *ifEntry->ifType == IF_MIB_IFTYPE_FASTETHERFX
-		|| *ifEntry->ifType == IF_MIB_IFTYPE_GIGABITETHERNET)) {
+	    && ianaiftype_mib_proc_isether(*ifEntry->ifType)) {
 	    name = fmt_ether_address(ipNetToMediaEntry->ipNetToMediaPhysAddress, SCLI_FMT_NAME);
 	    if (name) {
 		g_string_sprintfa(s, " (%s)", name);
@@ -601,7 +622,7 @@ show_ip_mapping(scli_interp_t *interp, int argc, char **argv)
 static void
 fmt_ip_info(GString *s, ip_mib_ip_t *ip)
 {
-    int const indent = 16;
+    int const indent = 18;
     const char *e;
 
     e = fmt_enum(forwarding, ip->ipForwarding);
@@ -610,18 +631,61 @@ fmt_ip_info(GString *s, ip_mib_ip_t *ip)
     }
 
     if (ip->ipDefaultTTL) {
-	g_string_sprintfa(s, "%-*s%d hops\n", indent, "Default TTL:",
+	g_string_sprintfa(s, "%-*s%d hops\n", indent, "Default-TTL:",
 			  *ip->ipDefaultTTL);
     }
 
     if (ip->ipReasmTimeout) {
-	g_string_sprintfa(s, "%-*s%d seconds\n", indent, "Reasm Timeout:",
+	g_string_sprintfa(s, "%-*s%d seconds\n", indent, "Reasm-Timeout:",
 			  *ip->ipReasmTimeout);
     }
 }
 
 
 
+static void
+xml_ip_info(xmlNodePtr tree, ip_mib_ip_t *ip)
+{
+    xmlNodePtr node;
+    const char *e;
+
+    e = fmt_enum(forwarding, ip->ipForwarding);
+    if (e) {
+	(void) xml_new_child(tree, NULL, "forwarding", "%s", e);
+    }
+
+    if (ip->ipDefaultTTL) {
+	node = xml_new_child(tree, NULL, "default-ttl", "%d",
+			     *ip->ipDefaultTTL);
+	xml_set_prop(node, "unit", "hops");
+    }
+
+    if (ip->ipReasmTimeout) {
+	node = xml_new_child(tree, NULL, "reassemble-timeout", "%d",
+			     *ip->ipDefaultTTL);
+	xml_set_prop(node, "unit", "seconds");
+    }
+}
+
+
+
+static void
+xxx_ip_info(GString *s, xmlNodePtr tree)
+{
+    int const indent = 18;
+    xmlNodePtr node;
+
+    for (node = tree->children; node; node = node->next) {
+	g_printerr("%s\n", xmlGetNodePath(node));
+	if (xmlStrcmp(node->name, (const xmlChar *) "forwarding") == 0) {
+	    g_string_sprintfa(s, "%-*s%s\n", indent, "Forwarding:", 
+			      node->content);
+	}
+    }
+}
+
+
+
 static int
 show_ip_info(scli_interp_t *interp, int argc, char **argv)
 {
@@ -643,7 +707,16 @@ show_ip_info(scli_interp_t *interp, int argc, char **argv)
     }
 
     if (ip) {
-	fmt_ip_info(interp->result, ip);
+	if (scli_interp_xml(interp)) {
+	    xml_ip_info(interp->xml_node, ip);
+	} else {
+	    fmt_ip_info(interp->result, ip);
+	}
+    }
+
+    if (scli_interp_xml(interp)) {
+	xxx_ip_info(interp->result, interp->xml_node);
+	g_printerr(interp->result->str);
     }
 
     if (ip) ip_mib_free_ip(ip);
@@ -665,7 +738,7 @@ set_ip_forwarding(scli_interp_t *interp, int argc, char **argv)
 	return SCLI_SYNTAX_NUMARGS;
     }
 
-    if (! gsnmp_enum_get_number(forwarding, argv[1], &value)) {
+    if (! gnet_snmp_enum_get_number(forwarding, argv[1], &value)) {
 	g_string_assign(interp->result, argv[1]);
 	return SCLI_SYNTAX_VALUE;
     }
@@ -791,8 +864,8 @@ scli_init_ip_mode(scli_interp_t *interp)
 	  "The `show ip info' command displays parameters of the IP\n"
 	  "protocol engine, such as the default TTL or whether the\n"
 	  "node is forwarding IP packets.",
-	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_DRY,
-	  NULL, NULL,
+	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_XML | SCLI_CMD_FLAG_DRY,
+	  "ip info", NULL,
 	  show_ip_info },
 	
 	{ "show ip forwarding", NULL,
@@ -814,10 +887,11 @@ scli_init_ip_mode(scli_interp_t *interp)
 	  "assigned to network interfaces. The command generates a table\n"
 	  "with the following columns:\n"
 	  "\n"
-	  "  INTERFACE network interface number\n"
-	  "  ADDRESS   IP address\n"
-	  "  PREFIX    IP address prefix length\n"
-	  "  NAME      name of the IP address",
+	  "  ADDRESS     IP address\n"
+	  "  PREFIX      IP address prefix length\n"
+	  "  NAME        name of the IP address\n"
+	  "  INTERFACE   network interface number\n"
+	  "  DESCRIPTION description of the network interface",
 	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_XML | SCLI_CMD_FLAG_DRY,
 	  "ip", NULL,
 	  show_ip_addresses },
