@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software Foundation,
  * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  * 
- * @(#) $Id: monitor.c,v 1.18 2002/07/02 14:17:16 schoenw Exp $
+ * @(#) $Id: monitor.c 2101 2007-01-02 22:16:41Z schoenw $
  */
 
 #include "scli.h"
@@ -83,7 +83,7 @@ fix_string(guchar *s, guint16 *len)
 
 
 static void
-snmp_decode_hook(GSList *list)
+snmp_decode_hook(GList *list)
 {
     static char x[] = { '-', '/', '-', '\\', '|' };
     static int i = 0;
@@ -179,7 +179,7 @@ prompt(char *message)
 
 
 static int
-show_system(GSnmpSession *peer, int flags)
+show_system(GNetSnmp *peer, int flags)
 {
     snmpv2_mib_system_t *system = NULL;
     static struct timeval last, now;
@@ -206,10 +206,17 @@ show_system(GSnmpSession *peer, int flags)
     }
     snmpv2_mib_get_system(peer, &system, mask);
     if (peer->error_status) {
+	GURI *uri;
+	gchar *name = NULL;
+	uri = gnet_snmp_get_uri(peer);
+	if (uri) {
+	    name = gnet_uri_get_string(uri);
+	}
         move(0, 10);
         clrtoeol();
-        mvprintw(0, 10, "%s:%d", peer->taddress, peer->port);
+        mvprintw(0, 10, "%s", name ? name : "?");
         mvaddstr(0, (int) (COLS-strlen(timestr)-1), timestr);
+	if (name) g_free(name);
         return STOP_FLAG_SNMP_FAILURE;
     }
 
@@ -218,6 +225,8 @@ show_system(GSnmpSession *peer, int flags)
 	if (system->sysUpTime) {
 	    unsigned days, hours, minutes, seconds;
 	    guint32 secs;
+	    gchar *name = NULL;
+	    GURI *uri;
 	    if (last.tv_sec && last.tv_usec) {
 		if (sysUpTime > *(system->sysUpTime)) {
 		    return STOP_FLAG_RESTART;
@@ -230,11 +239,16 @@ show_system(GSnmpSession *peer, int flags)
 	    seconds = secs % 60;
 	    move(0, 10);
 	    clrtoeol();
-	    mvprintw(0, 10, "%s:%d up %d %s %02d:%02d:%02d",
-		     peer->taddress, peer->port,
+	    uri = gnet_snmp_get_uri(peer);
+	    if (uri) {
+		name = gnet_uri_get_string(uri);
+	    }
+	    mvprintw(0, 10, "%s up %d %s %02d:%02d:%02d",
+		     name ? name : "?",
 		     days, (days == 1) ? "day" : "days",
 		     hours, minutes, seconds);
 	    sysUpTime = *(system->sysUpTime);
+	    if (name) g_free(name);
 	}
 	if (system->sysDescr) {
 	    fix_string(system->sysDescr, &(system->_sysDescrLength));
@@ -270,7 +284,7 @@ show_system(GSnmpSession *peer, int flags)
 
 
 static void
-show_ip(GSnmpSession *peer, int flags)
+show_ip(GNetSnmp *peer, int flags)
 {
     ip_mib_ip_t *ip = NULL;
     static guint32 ipInReceives = 0;
@@ -280,7 +294,11 @@ show_ip(GSnmpSession *peer, int flags)
     static guint32 ipOutSent = 0;
     static struct timeval last, now;
     double delta;
-    int mask = 0;
+    int mask = IP_MIB_IPFORWDATAGRAMS | IP_MIB_IPINRECEIVES |
+	    IP_MIB_IPOUTREQUESTS | IP_MIB_IPOUTDISCARDS |
+	    IP_MIB_IPOUTNOROUTES | IP_MIB_IPFRAGOKS |
+	    IP_MIB_IPFRAGFAILS | IP_MIB_IPFRAGCREATES |
+	    IP_MIB_IPREASMOKS;
 
     if (flags & STOP_FLAG_RESTART) {
         last.tv_sec = last.tv_usec = 0;
@@ -363,7 +381,7 @@ show_ip(GSnmpSession *peer, int flags)
 
 
 static void
-show_udp(GSnmpSession *peer, int flags)
+show_udp(GNetSnmp *peer, int flags)
 {
     udp_mib_udp_t *udp = NULL;
     static guint32 udpInDatagrams = 0;
@@ -409,7 +427,7 @@ show_udp(GSnmpSession *peer, int flags)
 
 
 static void
-show_tcp(GSnmpSession *peer, int flags)
+show_tcp(GNetSnmp *peer, int flags)
 {
     tcp_mib_tcp_t *tcp = NULL;
     static guint32 tcpInSegs = 0;
@@ -481,13 +499,21 @@ show_tcp(GSnmpSession *peer, int flags)
 
 
 static void
-show_interior(GSnmpSession *peer)
+show_interior(GNetSnmp *peer)
 {
     int y = 0;
+    GURI *uri;
+    gchar *name = NULL;
+
+    uri = gnet_snmp_get_uri(peer);
+    if (uri) {
+	name = gnet_uri_get_string(uri);
+    }
     
     clear();
     y = 0;
-    mvprintw(y, 10, "%s:%d", peer->taddress, peer->port);
+    mvprintw(y, 10, "%s", name ? name : "?");
+    if (name) g_free(name); name = NULL;
     mvaddstr(y++, 0, "Agent:");
     sys_des_line = y;
     mvaddstr(y++, 0, "Descr:");
@@ -555,7 +581,7 @@ help(scli_cmd_t *cmd, int argc, char **argv)
 
 
 static int
-mainloop(scli_interp_t *interp, scli_cmd_t *cmd, int argc, char **argv)
+mainloop_curses(scli_interp_t *interp, scli_cmd_t *cmd, int argc, char **argv)
 {
     int c, num, flags = 0;
     char *input, buffer[80], *end;
@@ -604,8 +630,8 @@ mainloop(scli_interp_t *interp, scli_cmd_t *cmd, int argc, char **argv)
 	if (flags & STOP_FLAG_SNMP_FAILURE) {
 	    const char *error = NULL;
 	    if (interp->peer && interp->peer->error_status) {
-		error = gsnmp_enum_get_label(gsnmp_enum_error_table,
-					     interp->peer->error_status);
+		error = gnet_snmp_enum_get_label(gnet_snmp_enum_error_table,
+						 interp->peer->error_status);
 	    }
 	    scli_interp_reset(interp);
 	    g_string_assign(interp->header, "NO DATA AVAILABLE");
@@ -708,13 +734,14 @@ mainloop(scli_interp_t *interp, scli_cmd_t *cmd, int argc, char **argv)
 
 
 int
-scli_monitor(scli_interp_t *interp, scli_cmd_t *cmd, int argc, char **argv)
+scli_monitor_curses(scli_interp_t *interp, scli_cmd_t *cmd,
+		    int argc, char **argv)
 {
     int code = SCLI_OK;
 
     scli_curses_on();
     g_snmp_list_decode_hook = snmp_decode_hook;
-    code = mainloop(interp, cmd, argc, argv);
+    code = mainloop_curses(interp, cmd, argc, argv);
     if (code == SCLI_OK || code == SCLI_EXIT) {
 	scli_interp_reset(interp);
     }
@@ -723,6 +750,39 @@ scli_monitor(scli_interp_t *interp, scli_cmd_t *cmd, int argc, char **argv)
 }
 
 
+
+int
+scli_monitor_proto(scli_interp_t *interp, scli_cmd_t *cmd,
+		   int argc, char **argv)
+{
+    int flags = 0;
+    int code = SCLI_OK;
+
+    flags |= STOP_FLAG_RESTART;
+    flags |= STOP_FLAG_NODELAY;
+    
+    while (! (flags & STOP_FLAG_DONE)) {
+	if (! (flags & STOP_FLAG_SNMP_FAILURE)) {
+	    scli_interp_reset(interp);
+	    if (! (interp->flags & SCLI_INTERP_FLAG_DRY)
+		|| ((interp->flags & SCLI_INTERP_FLAG_DRY) && (cmd->flags & SCLI_CMD_FLAG_DRY))) {
+		code = (cmd->func) (interp, argc, argv);
+	    } else {
+		code = SCLI_OK;
+	    }
+	    if (code != SCLI_OK) {
+		break;
+	    }
+	    scli_show_results(interp, cmd, code);
+	    g_printerr("** sleeping (%d)...\n", code);
+	    sleep(1);
+	}
+    }
+    return code;
+}
+    
+
+
 struct loop_data {
     scli_interp_t *interp;
     scli_cmd_t *cmd;
@@ -739,14 +799,14 @@ loop_iteration(gpointer data)
     scli_cmd_t *cmd = loop_data->cmd;
     int argc = loop_data->argc;
     char **argv = loop_data->argv;
+    int code = SCLI_OK;
     
-    g_printerr("loop_iteration()\n");
     scli_interp_reset(interp);
     if (! (interp->flags & SCLI_INTERP_FLAG_DRY)
 	|| ((interp->flags & SCLI_INTERP_FLAG_DRY) && (cmd->flags & SCLI_CMD_FLAG_DRY))) {
-	(void) (cmd->func) (interp, argc, argv);
+	code = (cmd->func) (interp, argc, argv);
     }
-    g_print("%s\n%s", interp->header->str, interp->result->str);
+    scli_show_results(interp, cmd, code);
     return TRUE;
 }
 
@@ -756,7 +816,6 @@ static gboolean
 loop_input(GIOChannel *source, GIOCondition condition, gpointer data)
 {
     GMainLoop *loop = (GMainLoop *) data;
-    g_printerr("loop_input()\n");
     g_main_quit(loop);
     return TRUE;
 }
@@ -788,7 +847,7 @@ scli_loop(scli_interp_t *interp, scli_cmd_t *cmd, int argc, char **argv)
 
     loop = g_main_new(TRUE);
     
-    g_timeout_add(1000, loop_iteration, &loop_data);
+    g_timeout_add(5000, loop_iteration, &loop_data);
 
     /*
     (void) initscr();
