@@ -17,13 +17,22 @@
  * along with this program; if not, write to the Free Software Foundation,
  * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  * 
- * @(#) $Id: netsnmp.c,v 1.7 2002/07/19 09:17:16 schoenw Exp $
+ * @(#) $Id: netsnmp.c 717 2004-10-14 21:16:08Z schoenw $
  */
 
 #include "scli.h"
 
 #include "ucd-snmp-mib.h"
 
+#define UCD_SNMP_MIB_VERSIONDODEBUGGING_DISABLED 0
+#define UCD_SNMP_MIB_VERSIONDODEBUGGING_ENABLED  1
+
+static GNetSnmpEnum const ucd_snmp_mib_enums_versionDoDebugging[] = {
+    { UCD_SNMP_MIB_VERSIONDODEBUGGING_DISABLED, "disabled" },
+    { UCD_SNMP_MIB_VERSIONDODEBUGGING_ENABLED,  "enabled" },
+    { 0, NULL }
+};
+
 
 static void
 fmt_info(GString *s, ucd_snmp_mib_version_t *version)
@@ -285,8 +294,8 @@ set_netsnmp_debugging(scli_interp_t *interp, int argc, char **argv)
 	return SCLI_SYNTAX_NUMARGS;
     }
 
-    if (! gsnmp_enum_get_number(ucd_snmp_mib_enums_versionDoDebugging,
-				argv[1], &value)) {
+    if (! gnet_snmp_enum_get_number(ucd_snmp_mib_enums_versionDoDebugging,
+				    argv[1], &value)) {
 	g_string_assign(interp->result, argv[1]);
 	return SCLI_SYNTAX_VALUE;
     }
