@@ -20,7 +20,7 @@
  * along with this program; if not, write to the Free Software Foundation,
  * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  * 
- * @(#) $Id: scli.h,v 1.73 2002/07/22 19:44:56 schoenw Exp $
+ * @(#) $Id: scli.h 2094 2007-01-02 17:15:11Z schoenw $
  */
 
 #ifndef _SCLI_H
@@ -48,11 +48,11 @@
 
 #include <libxml/xmlmemory.h>
 #include <libxml/tree.h>
+#include <libxml/xpath.h>
 
 #include <glib.h>
 #include <gmodule.h>
-
-#include "g_snmp.h"
+#include <gsnmp.h>
 
 
 extern char const scli_copyright[];	/* copyright message (surprise) */
@@ -68,7 +68,7 @@ extern char const scli_copyright[];	/* copyright message (surprise) */
 #define TV_DELTA	0.1
 
 /*
- * The return codes used by the scli commands functions.
+ * The return codes used by the scli command functions.
  *
  * Theory of scli return codes:
  *
@@ -169,21 +169,24 @@ struct scli_alarm {
 };
 
 
-#define SCLI_INTERP_FLAG_INTERACTIVE	0x01
-#define SCLI_INTERP_FLAG_RECURSIVE	0x02
-#define SCLI_INTERP_FLAG_MONITOR	0x04
-#define SCLI_INTERP_FLAG_LOOP		0x08
-#define SCLI_INTERP_FLAG_XML		0x10
-#define SCLI_INTERP_FLAG_DRY		0x20
-#define SCLI_INTERP_FLAG_PROTO		0x40
-#define SCLI_INTERP_FLAG_QUIET		0x80
+#define SCLI_INTERP_FLAG_INTERACTIVE	0x001
+#define SCLI_INTERP_FLAG_RECURSIVE	0x002
+#define SCLI_INTERP_FLAG_MONITOR	0x004
+#define SCLI_INTERP_FLAG_LOOP		0x008
+#define SCLI_INTERP_FLAG_XML		0x010
+#define SCLI_INTERP_FLAG_FMT		0x020
+#define SCLI_INTERP_FLAG_DRY		0x040
+#define SCLI_INTERP_FLAG_PROTO		0x080
+#define SCLI_INTERP_FLAG_QUIET		0x100
+#define SCLI_INTERP_FLAG_NOLOOKUP	0x200
 
 struct scli_interp {
     char *name;			/* name of the interpreter */
     GNode *cmd_root;		/* root of the command tree */
     GSList *mode_list;		/* list of registered modes */
     GSList *alias_list;		/* list of command aliases */
-    GSList *interp_list;	/* list of slave interpreters */
+    GSList *slave_list;		/* list of slave interpreters */
+    struct scli_interp *master;	/* master interpreter (if any) */
     GSList *alarm_list;		/* list of detected alarms */
     int	flags;			/* interpreter flags */
     GString *result;		/* string result buffer */
@@ -191,14 +194,17 @@ struct scli_interp {
     xmlDocPtr xml_doc;		/* xml document if we produce xml output */
     xmlNodePtr xml_node;	/* current xml node in the xml document */
     char *pager;		/* external pager we are using */
-    GSnmpSession *peer;		/* snmp peer we are talking to */
+    GNetSnmp *peer;		/* snmp session we are using */
     gint delay;			/* delay between updates in milliseconds */
-    int port;			/* default port number, defaults to 161 */
+    GNetSnmpTDomain tdomain;	/* transport domain */
+    GInetAddr *taddress;	/* transport address */
     int snmp;			/* version of the preferred SNMP protocol */
     time_t epoch;		/* epoch (used to invalidate cached data) */
     int xid;			/* unique identifier for each transaction */
     time_t xtime;		/* transaction time */
     int regex_flags;		/* regular expression flags (see regcomp(3)) */
+    guint retries;		/* number of retries */
+    guint timeout;		/* timeout causing a retry in ms */
 };
 
 extern scli_interp_t *
@@ -227,6 +233,8 @@ scli_interp_reset(scli_interp_t *interp);
 	(interp->flags & SCLI_INTERP_FLAG_PROTO)
 #define scli_interp_quiet(interp) \
 	(interp->flags & SCLI_INTERP_FLAG_QUIET)
+#define scli_interp_nolookup(interp) \
+	(interp->flags & SCLI_INTERP_FLAG_NOLOOKUP)
 
 extern scli_alarm_t *
 scli_alarm_create(scli_interp_t *interp, char *desc);
@@ -261,6 +269,9 @@ scli_eval_file_stream(scli_interp_t *interp, FILE *stream);
 extern int
 scli_eval_init_file(scli_interp_t *interp);
 
+extern void
+scli_show_results(scli_interp_t *interp, scli_cmd_t *cmd, int code);
+
 extern void
 scli_register_mode(scli_interp_t *interp, scli_mode_t *mode);
 
@@ -268,8 +279,8 @@ extern int
 scli_create_command(scli_interp_t *interp, scli_cmd_t *cmd);
 
 extern int
-scli_open_community(scli_interp_t *interp, char *host, int port,
-		    char *community);
+scli_open_community(scli_interp_t *interp, GNetSnmpTDomain tdomain,
+		    GInetAddr *addr, char *community);
 extern void
 scli_close(scli_interp_t *interp);
 
@@ -288,6 +299,12 @@ scli_curses_off(void);
 extern int
 scli_set_pager(scli_interp_t *interp, const char *pager);
 
+extern int
+scli_set_retries(scli_interp_t *interp, const guint retries);
+
+extern int
+scli_set_timeout(scli_interp_t *interp, guint timeout);
+
 extern void
 scli_snmp_error(scli_interp_t *interp);
 
@@ -358,6 +375,9 @@ scli_init_netsnmp_mode(scli_interp_t *interp);
 extern void
 scli_init_3com_mode(scli_interp_t *interp);
 
+extern void
+scli_init_hp_mode(scli_interp_t *interp);
+
 /* 
  * A data structure used to obtain vendor information from the
  * enterprise OID. See the file vendors.c for details.
@@ -373,7 +393,7 @@ extern const scli_vendor_t*
 scli_get_vendor(guint32 enterp);
 
 extern const scli_vendor_t*
-scli_get_vendor_oid(guint32 oid[], gsize len);
+scli_get_vendor_oid(guint32 oid[], guint16 len);
 
 extern const scli_vendor_t*
 scli_get_ieee_vendor(guint32 prefix);
@@ -383,9 +403,15 @@ scli_get_ieee_vendor(guint32 prefix);
  * modes.
  */
 
+extern const char *
+xml_timeticks(guint32 timeticks);
+
 extern const char *
 fmt_timeticks(guint32 timeticks);
 
+extern char const *
+xml_date_and_time(guchar *data, gsize len);
+
 extern char const *
 fmt_date_and_time(guchar *data, gsize len);
 
@@ -396,10 +422,14 @@ extern char const *
 fmt_seconds(guint32 seconds);
 
 extern char const *
-fmt_enum(GSnmpEnum const *table, gint32 *number);
+fmt_enum(GNetSnmpEnum const *table, gint32 *number);
 
 extern void
-xxx_enum(GString *s, int width, GSnmpEnum const *table, gint32 *number);
+xxx_enum(GString *s, int width, GNetSnmpEnum const *table, gint32 *number);
+
+extern char const *
+fmt_identity(GNetSnmpIdentity const *table,
+	     guint32 const *oid, gsize oidlen);
 
 extern char const *
 fmt_kmg(guint32 number);
@@ -432,6 +462,9 @@ fmt_counter_dt(GString *s, guint32 *new_counter, guint32 *old_counter,
 extern void
 fmt_indent_string(GString *s, int indent, char *label, int len, char *string);
 
+extern const char*
+xml_display_string(int len, char *string);
+
 extern void
 fmt_display_string(GString *s, int indent, char *label, int len, char *string);
 
@@ -456,7 +489,8 @@ scan_port_set(guchar *bits, gsize bits_len, char *string);
 
     
 /*
- * XML utilities.
+ * XML utilities that are quite handy and used frequently by scli
+ * modes.
  */
 
 extern void
