@@ -17,23 +17,26 @@
  * along with this program; if not, write to the Free Software Foundation,
  * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  * 
- * @(#) $Id: snmp.c,v 1.31 2002/07/19 09:17:16 schoenw Exp $
+ * @(#) $Id: snmp.c 739 2004-11-08 21:04:33Z schoenw $
  */
 
 #include "scli.h"
 
 #include "snmpv2-mib.h"
 #include "snmp-framework-mib.h"
-#include "snmp-view-based-acm-mib.h"
-#include "snmp-user-based-sm-mib.h"
 #include "snmp-target-mib.h"
 #include "snmp-notification-mib.h"
-
+#include "snmp-community-mib.h"
 #include "snmp-view-based-acm-mib-proc.h"
 #include "snmp-user-based-sm-mib-proc.h"
+#include "notification-log-mib.h"
+
+#include "if-mib.h"
+#include "snmpv2-mib.h"
+#include "entity-mib.h"
 
 
-static GSnmpEnum const mp_model[] = {
+static GNetSnmpEnum const mp_model[] = {
     { 0,	"SNMPv1" },
     { 1,	"SNMPv2c" },
     { 2,	"SNMPv2u/*" },
@@ -43,7 +46,7 @@ static GSnmpEnum const mp_model[] = {
 
 
 
-static GSnmpEnum const security_model[] = {
+static GNetSnmpEnum const security_model[] = {
     { 0,	"any" },
     { 1,	"v1" },
     { 2,	"v2c" },
@@ -53,16 +56,16 @@ static GSnmpEnum const security_model[] = {
 
 
 
-static GSnmpEnum const security_level[] = {
-    { SNMP_VIEW_BASED_ACM_MIB_VACMACCESSSECURITYLEVEL_NOAUTHNOPRIV,	"--" },
-    { SNMP_VIEW_BASED_ACM_MIB_VACMACCESSSECURITYLEVEL_AUTHNOPRIV,	"a-" },
-    { SNMP_VIEW_BASED_ACM_MIB_VACMACCESSSECURITYLEVEL_AUTHPRIV,		"ap" },
+static GNetSnmpEnum const security_level[] = {
+    { SNMP_FRAMEWORK_MIB_SNMPSECURITYLEVEL_NOAUTHNOPRIV,	"--" },
+    { SNMP_FRAMEWORK_MIB_SNMPSECURITYLEVEL_AUTHNOPRIV,		"a-" },
+    { SNMP_FRAMEWORK_MIB_SNMPSECURITYLEVEL_AUTHPRIV,		"ap" },
     { 0, NULL }
 };
 
 
 
-static GSnmpEnum const view_tree_family_type[] = {
+static GNetSnmpEnum const view_tree_family_type[] = {
     { SNMP_VIEW_BASED_ACM_MIB_VACMVIEWTREEFAMILYTYPE_INCLUDED,	"incl" },
     { SNMP_VIEW_BASED_ACM_MIB_VACMVIEWTREEFAMILYTYPE_EXCLUDED,	"excl" },
     { 0, NULL }
@@ -81,7 +84,7 @@ static guint32 const usmNoPrivProtocol[]
 static guint32 const usmDESPrivProtocol[]
 	= { SNMP_USER_BASED_SM_MIB_USMDESPRIVPROTOCOL };
 
-GSnmpIdentity const sec_proto_identities[] = {
+GNetSnmpIdentity const sec_proto_identities[] = {
     { usmNoAuthProtocol,
       sizeof(usmNoAuthProtocol)/sizeof(guint32),
       "none" },
@@ -135,7 +138,7 @@ create_snmp_vacm_member(scli_interp_t *interp, int argc, char **argv)
     }
 
     if (argc == 4) {
-	if (! gsnmp_enum_get_number(security_model, argv[3], &model)) {
+	if (! gnet_snmp_enum_get_number(security_model, argv[3], &model)) {
 	    g_string_assign(interp->result, argv[3]);
 	    return SCLI_SYNTAX_VALUE;
 	}
@@ -186,7 +189,7 @@ delete_snmp_vacm_member(scli_interp_t *interp, int argc, char **argv)
     }
 
     if (argc == 4) {
-	if (! gsnmp_enum_get_number(security_model, argv[3], &model)) {
+	if (! gnet_snmp_enum_get_number(security_model, argv[3], &model)) {
 	    if (regex_name) regfree(regex_name);
 	    if (regex_group) regfree(regex_group);
 	    g_string_assign(interp->result, argv[3]);
@@ -291,7 +294,8 @@ create_snmp_usm_user(scli_interp_t *interp, int argc, char **argv)
 static void
 fmt_snmp_engine(GString *s,
 		snmp_framework_mib_snmpEngine_t *snmpEngine,
-		snmpv2_mib_snmp_t *snmp)
+		snmpv2_mib_snmp_t *snmp,
+		notification_log_mib_nlmConfig_t *nlmConfig)
 {
     int const indent = 22;
     int i;
@@ -301,7 +305,7 @@ fmt_snmp_engine(GString *s,
 	if (snmpEngine->snmpEngineID) {
 	    guint32 enterp = 0;
 	    const scli_vendor_t *vendor;
-	    g_string_sprintfa(s, "%-*s", indent, "Engine Identifier:");
+	    g_string_sprintfa(s, "%-*s", indent, "Engine-Identifier:");
 	    for (i = 0; i < snmpEngine->_snmpEngineIDLength; i++) {
 		g_string_sprintfa(s, "%02x", snmpEngine->snmpEngineID[i]);
 	    }
@@ -312,7 +316,7 @@ fmt_snmp_engine(GString *s,
 	    enterp = (enterp << 8) + snmpEngine->snmpEngineID[3];
 	    vendor = scli_get_vendor(enterp);
 	    if (vendor && vendor->name) {
-		g_string_sprintfa(s, "%-*s%s", indent, "Engine Vendor:",
+		g_string_sprintfa(s, "%-*s%s", indent, "Engine-Vendor:",
 				  vendor->name);
 		if (vendor->url) {
 		    g_string_sprintfa(s, " <%s>", vendor->url);
@@ -322,17 +326,17 @@ fmt_snmp_engine(GString *s,
 	}
 	if (snmpEngine->snmpEngineTime) {
 	    g_string_sprintfa(s, "%-*s%u seconds\n",
-			      indent, "Engine Uptime:",
+			      indent, "Engine-Uptime:",
 			      *(snmpEngine->snmpEngineTime));
 	}
 	if (snmpEngine->snmpEngineBoots) {
 	    g_string_sprintfa(s, "%-*s%u\n",
-			      indent, "Number of Boots:",
+			      indent, "Number-of-Boots:",
 			      *(snmpEngine->snmpEngineBoots));
 	}
 	if (snmpEngine->snmpEngineMaxMessageSize) {
-	    g_string_sprintfa(s, "%-*s%u byte\n",
-			      indent, "Maximum Message Size:",
+	    g_string_sprintfa(s, "%-*s%u bytes\n",
+			      indent, "Maximum-Message-Size:",
 			      *(snmpEngine->snmpEngineMaxMessageSize));
 	}
     }
@@ -341,7 +345,19 @@ fmt_snmp_engine(GString *s,
 		     snmp->snmpEnableAuthenTraps);
 	if (e) {
 	    g_string_sprintfa(s, "%-*s%s\n",
-			      indent, "Authentication Traps:", e);
+			      indent, "Authentication-Traps:", e);
+	}
+    }
+    if (nlmConfig) {
+	if (nlmConfig->nlmConfigGlobalEntryLimit) {
+	    g_string_sprintfa(s, "%-*s%u\n",
+			      indent, "Notification-Log-Limit:",
+			      *nlmConfig->nlmConfigGlobalEntryLimit);
+	}
+	if (nlmConfig->nlmConfigGlobalEntryLimit) {
+	    g_string_sprintfa(s, "%-*s%u minutes\n",
+			      indent, "Notification-Log-Ageout:",
+			      *nlmConfig->nlmConfigGlobalEntryLimit);
 	}
     }
 }
@@ -353,6 +369,7 @@ show_snmp_engine(scli_interp_t *interp, int argc, char **argv)
 {
     snmp_framework_mib_snmpEngine_t *snmpEngine;
     snmpv2_mib_snmp_t *snmp;
+    notification_log_mib_nlmConfig_t *nlmConfig;
     
     g_return_val_if_fail(interp, SCLI_ERROR);
 
@@ -369,11 +386,13 @@ show_snmp_engine(scli_interp_t *interp, int argc, char **argv)
 	return SCLI_SNMP;
     }
     snmpv2_mib_get_snmp(interp->peer, &snmp, SNMPV2_MIB_SNMPENABLEAUTHENTRAPS);
+    notification_log_mib_get_nlmConfig(interp->peer, &nlmConfig, 0);
 
-    fmt_snmp_engine(interp->result, snmpEngine, snmp);
+    fmt_snmp_engine(interp->result, snmpEngine, snmp, nlmConfig);
     
     if (snmpEngine) snmp_framework_mib_free_snmpEngine(snmpEngine);
     if (snmp) snmpv2_mib_free_snmp(snmp);
+    if (nlmConfig) notification_log_mib_free_nlmConfig(nlmConfig);
     
     return SCLI_OK;
 }
@@ -386,13 +405,13 @@ fmt_snmp_resource(GString *s,
 		  snmpv2_mib_system_t *system)
 {
     int i;
-    int const indent = 14;
+    int const indent = 18;
     
-    g_string_sprintfa(s, "%-*s%d\n", indent, "MIB Resource:",
+    g_string_sprintfa(s, "%-*s%d\n", indent, "MIB-Resource:",
 		      sysOREntry->sysORIndex);
     if (sysOREntry->sysORUpTime && system && system->sysUpTime) {
 	guint32 dsecs = *system->sysUpTime - *sysOREntry->sysORUpTime;
-	g_string_sprintfa(s, "%-*s%s\n", indent, "Last Change:",
+	g_string_sprintfa(s, "%-*s%s\n", indent, "Last-Change:",
 			  fmt_timeticks(dsecs));
     }
     
@@ -463,8 +482,8 @@ fmt_snmp_vacm_member(GString *s,
     fmt_storage_type(s, vacmGroupEntry->vacmSecurityToGroupStorageType);
     fmt_row_status(s, vacmGroupEntry->vacmSecurityToGroupStatus);
     
-    model = gsnmp_enum_get_label(security_model,
-				 vacmGroupEntry->vacmSecurityModel);
+    model = gnet_snmp_enum_get_label(security_model,
+				     vacmGroupEntry->vacmSecurityModel);
     g_string_sprintfa(s, "  %-3s %-*.*s",
 		      model ? model : "", sec_name_width,
 		      (int) vacmGroupEntry->_vacmSecurityNameLength,
@@ -534,13 +553,13 @@ fmt_snmp_vacm_access(GString *s,
     fmt_storage_type(s, vacmAccessEntry->vacmAccessStorageType);
     fmt_row_status(s, vacmAccessEntry->vacmAccessStatus);
 
-    model = gsnmp_enum_get_label(security_model,
-				vacmAccessEntry->vacmAccessSecurityModel);
-    level = gsnmp_enum_get_label(security_level,
-				vacmAccessEntry->vacmAccessSecurityLevel);
+    model = gnet_snmp_enum_get_label(security_model,
+				     vacmAccessEntry->vacmAccessSecurityModel);
+    level = gnet_snmp_enum_get_label(security_level,
+				     vacmAccessEntry->vacmAccessSecurityLevel);
     if (vacmAccessEntry->vacmAccessContextMatch) {
-	match = gsnmp_enum_get_label(snmp_view_based_acm_mib_enums_vacmAccessContextMatch,
-				    *vacmAccessEntry->vacmAccessContextMatch);
+	match = gnet_snmp_enum_get_label(snmp_view_based_acm_mib_enums_vacmAccessContextMatch,
+					 *vacmAccessEntry->vacmAccessContextMatch);
     }
         
     g_string_sprintfa(s, "  %-*.*s", group_width,
@@ -658,7 +677,7 @@ fmt_snmp_vacm_view(GString *s,
 			  (int) vacmViewEntry->_vacmViewTreeFamilyViewNameLength,
 			  vacmViewEntry->vacmViewTreeFamilyViewName);
 
-	type = gsnmp_enum_get_label(view_tree_family_type,
+	type = gnet_snmp_enum_get_label(view_tree_family_type,
 				   *vacmViewEntry->vacmViewTreeFamilyType);
 	g_string_sprintfa(s, " %-4s ", type ? type : "");
 
@@ -730,7 +749,7 @@ static void
 fmt_snmp_usm_user(GString *s, snmp_user_based_sm_mib_usmUserEntry_t *usmUserEntry,
 		  int user_width, int sec_width)
 {
-    const char *label;
+    const char *e;
     
     fmt_storage_type(s, usmUserEntry->usmUserStorageType);
     fmt_row_status(s, usmUserEntry->usmUserStatus);
@@ -747,19 +766,15 @@ fmt_snmp_usm_user(GString *s, snmp_user_based_sm_mib_usmUserEntry_t *usmUserEntr
 	g_string_sprintfa(s, " %*s", sec_width, "");
     }
 
-    label = gsnmp_identity_get_label(sec_proto_identities,
-				    usmUserEntry->usmUserAuthProtocol,
-				    usmUserEntry->_usmUserAuthProtocolLength);
-    if (label) {
-	g_string_sprintfa(s, " %-8s", label ? label : "");
-    }
+    e = fmt_identity(sec_proto_identities,
+		     usmUserEntry->usmUserAuthProtocol,
+		     usmUserEntry->_usmUserAuthProtocolLength);
+    g_string_sprintfa(s, " %-8s", e ? e : "");
 
-    label = gsnmp_identity_get_label(sec_proto_identities,
-				    usmUserEntry->usmUserPrivProtocol,
-				    usmUserEntry->_usmUserPrivProtocolLength);
-    if (label) {
-	g_string_sprintfa(s, " %-8s", label ? label : "");
-    }
+    e = fmt_identity(sec_proto_identities,
+		     usmUserEntry->usmUserPrivProtocol,
+		     usmUserEntry->_usmUserPrivProtocolLength);
+    g_string_sprintfa(s, " %-8s", e ? e : "");
 
     g_string_append(s, "\n");
 }
@@ -1095,6 +1110,403 @@ show_snmp_notification_targets(scli_interp_t *interp, int argc, char **argv)
 
 
 
+static void
+fmt_snmp_context(GString *s,
+		 entity_mib_entLogicalEntry_t *logEntEntry)
+{
+    int const indent = 22;
+    const char *e;
+
+    fmt_display_string(s, indent, "Description:",
+		       (int) logEntEntry->_entLogicalDescrLength,
+		       logEntEntry->entLogicalDescr);
+    
+    fmt_display_string(s, indent, "Community:",
+		       (int) logEntEntry->_entLogicalCommunityLength,
+		       logEntEntry->entLogicalCommunity);
+
+    e = fmt_tdomain(logEntEntry->entLogicalTDomain,
+		    logEntEntry->_entLogicalTDomainLength);
+    g_string_sprintfa(s, " %-6s", e ? e : "?");
+
+    e = fmt_taddress(logEntEntry->entLogicalTDomain,
+		     logEntEntry->_entLogicalTDomainLength,
+		     logEntEntry->entLogicalTAddress,
+		     logEntEntry->_entLogicalTAddressLength);
+    g_string_sprintfa(s, " %-*s", 32, e ? e : "?");
+    g_string_sprintfa(s, "\n");
+
+}
+
+
+    
+static int
+show_snmp_contexts(scli_interp_t *interp, int argc, char **argv)
+{
+    entity_mib_entLogicalEntry_t **logEntTable = NULL;
+    int i;
+    
+    g_return_val_if_fail(interp, SCLI_ERROR);
+
+    if (argc > 1) {
+	return SCLI_SYNTAX_NUMARGS;
+    }
+
+    if (scli_interp_dry(interp)) {
+	return SCLI_OK;
+    }
+
+    entity_mib_get_entLogicalTable(interp->peer, &logEntTable, 0);
+    if (interp->peer->error_status) {
+	return SCLI_SNMP;
+    }
+
+    if (logEntTable) {
+	for (i = 0; logEntTable[i]; i++) {
+	    fmt_snmp_context(interp->result, logEntTable[i]);
+	}
+    }
+
+    if (logEntTable) {
+	entity_mib_free_entLogicalTable(logEntTable);
+    }
+
+    return SCLI_OK;
+}
+
+
+
+static void
+fmt_snmp_csm_community(GString *s,
+		       snmp_community_mib_snmpCommunityEntry_t *snmpCommunityEntry)
+{
+    int const indent = 22;
+
+    fmt_display_string(s, indent, "Index:",
+		       (int) snmpCommunityEntry->_snmpCommunityIndexLength,
+		       snmpCommunityEntry->snmpCommunityIndex);
+    
+    fmt_display_string(s, indent, "Name:",
+		       (int) snmpCommunityEntry->_snmpCommunityNameLength,
+		       snmpCommunityEntry->snmpCommunityName);
+
+    fmt_display_string(s, indent, "SecurityName:",
+		       (int) snmpCommunityEntry->_snmpCommunitySecurityNameLength,
+		       snmpCommunityEntry->snmpCommunitySecurityName);
+
+    g_string_sprintfa(s, "\n");
+}
+
+
+    
+static int
+show_snmp_csm_communities(scli_interp_t *interp, int argc, char **argv)
+{
+    snmp_community_mib_snmpCommunityEntry_t **snmpCommunityTable;
+    int i;
+    
+    g_return_val_if_fail(interp, SCLI_ERROR);
+
+    if (argc > 1) {
+	return SCLI_SYNTAX_NUMARGS;
+    }
+
+    if (scli_interp_dry(interp)) {
+	return SCLI_OK;
+    }
+
+    snmp_community_mib_get_snmpCommunityTable(interp->peer, &snmpCommunityTable, 0);
+    if (interp->peer->error_status) {
+	return SCLI_SNMP;
+    }
+
+    if (snmpCommunityTable) {
+	for (i = 0; snmpCommunityTable[i]; i++) {
+	    fmt_snmp_csm_community(interp->result, snmpCommunityTable[i]);
+	}
+    }
+
+    if (snmpCommunityTable) {
+	snmp_community_mib_free_snmpCommunityTable(snmpCommunityTable);
+    }
+
+    return SCLI_OK;
+}
+
+
+
+static void
+fmt_snmp_notification_log_details_varbind(GString *s,
+		  notification_log_mib_nlmLogVariableEntry_t *nlmLogVarEntry)
+{
+    int const width = 18;
+    const char *e;
+    gchar buffer[42];
+    int i;
+    
+    if (! nlmLogVarEntry->nlmLogVariableID
+	|| !nlmLogVarEntry->_nlmLogVariableIDLength) {
+	return;
+    }
+
+    g_snprintf(buffer, sizeof(buffer), "Var-Name-%d:",
+	       nlmLogVarEntry->nlmLogVariableIndex);
+    g_string_sprintfa(s, "%-*s", width, buffer);
+    for (i = 0; i < nlmLogVarEntry->_nlmLogVariableIDLength; i++) {
+	g_string_sprintfa(s, "%s%d",
+			  i ? "." : "", nlmLogVarEntry->nlmLogVariableID[i]);
+    }
+    g_string_sprintfa(s, "\n");
+
+    e = fmt_enum(notification_log_mib_enums_nlmLogVariableValueType,
+		 nlmLogVarEntry->nlmLogVariableValueType);
+    if (e) {
+	g_snprintf(buffer, sizeof(buffer), "Var-Type-%d:",
+		   nlmLogVarEntry->nlmLogVariableIndex);
+	g_string_sprintfa(s, "%-*s%s\n", width, buffer, e);
+    }
+    
+    g_snprintf(buffer, sizeof(buffer), "Var-Value-%d:",
+	       nlmLogVarEntry->nlmLogVariableIndex);
+
+    if (nlmLogVarEntry->nlmLogVariableCounter32Val) {
+	g_string_sprintfa(s, "%-*s%u\n", width, buffer,
+			  *nlmLogVarEntry->nlmLogVariableCounter32Val);
+    } else if (nlmLogVarEntry->nlmLogVariableUnsigned32Val) {
+	g_string_sprintfa(s, "%-*s%u\n", width, buffer,
+			  *nlmLogVarEntry->nlmLogVariableUnsigned32Val);
+    } else if (nlmLogVarEntry->nlmLogVariableTimeTicksVal) {
+	g_string_sprintfa(s, "%-*s%u\n", width, buffer,
+			  *nlmLogVarEntry->nlmLogVariableTimeTicksVal);
+    } else if (nlmLogVarEntry->nlmLogVariableInteger32Val) {
+	g_string_sprintfa(s, "%-*s%d\n", width, buffer,
+			  *nlmLogVarEntry->nlmLogVariableInteger32Val);
+    } else if (nlmLogVarEntry->nlmLogVariableOctetStringVal) {
+	/* xxx */
+    } else if (nlmLogVarEntry->nlmLogVariableIpAddressVal) {
+	g_string_sprintfa(s, "%-*s%d.%d.%d.%d", width, buffer,
+			  nlmLogVarEntry->nlmLogVariableIpAddressVal[0],
+			  nlmLogVarEntry->nlmLogVariableIpAddressVal[1],
+			  nlmLogVarEntry->nlmLogVariableIpAddressVal[2],
+			  nlmLogVarEntry->nlmLogVariableIpAddressVal[3]);
+    } else if (nlmLogVarEntry->_nlmLogVariableOidValLength) {
+	/* xxx */
+    } else if (nlmLogVarEntry->nlmLogVariableCounter64Val) {
+	/* xxx */
+    } else if (nlmLogVarEntry->nlmLogVariableOpaqueVal) {
+	/* xxx */
+    } else {
+	g_string_sprintfa(s, "%-*s\n", width, buffer);
+    }
+}
+
+
+
+static void
+fmt_snmp_notification_log_details(GString *s,
+		  notification_log_mib_nlmLogEntry_t *nlmLogEntry,
+		  notification_log_mib_nlmLogVariableEntry_t **nlmLogVarTable)
+{
+    int const width = 18;
+    const char *e;
+    
+    g_string_sprintfa(s, "%-*s%.*s\n", width, "Log-Name:",
+		      nlmLogEntry->_nlmLogNameLength,
+		      nlmLogEntry->nlmLogName);
+    
+    g_string_sprintfa(s, "%-*s%d\n", width, "Log-Index:",
+		      nlmLogEntry->nlmLogIndex);
+
+    if (nlmLogEntry->nlmLogDateAndTime) {
+	g_string_sprintfa(s, "%-*s%s\n", width, "Time:",
+		  fmt_date_and_time(nlmLogEntry->nlmLogDateAndTime,
+				    nlmLogEntry->_nlmLogDateAndTimeLength));
+    } else if (nlmLogEntry->nlmLogTime) {
+	g_string_sprintfa(s, "%-*s%s\n", width, "Time:",
+			  fmt_timeticks(*(nlmLogEntry->nlmLogTime)));
+    }
+
+    e = fmt_taddress(nlmLogEntry->nlmLogEngineTDomain,
+		     nlmLogEntry->_nlmLogEngineTDomainLength,
+		     nlmLogEntry->nlmLogEngineTAddress,
+		     nlmLogEntry->_nlmLogEngineTAddressLength);
+    if (e) {
+	g_string_sprintfa(s, "%-*s%s\n", width, "Source:", e);
+    }
+
+    if (nlmLogEntry->nlmLogNotificationID) {
+	int i;
+	g_string_sprintfa(s, "%-*s", width, "Notification:");
+	for (i = 0; i < nlmLogEntry->_nlmLogNotificationIDLength; i++) {
+	    g_string_sprintfa(s, "%s%d",
+			      i ? "." : "", nlmLogEntry->nlmLogNotificationID[i]);
+	}
+	g_string_sprintfa(s, "\n");
+    }
+
+    if (nlmLogVarTable) {
+	int i;
+	for (i = 0; nlmLogVarTable[i]; i++) {
+	    if (nlmLogVarTable[i]->_nlmLogNameLength
+		== nlmLogEntry->_nlmLogNameLength
+		&& memcmp(nlmLogVarTable[i]->nlmLogName,
+			  nlmLogEntry->nlmLogName,
+			  nlmLogEntry->_nlmLogNameLength) == 0
+		&& nlmLogVarTable[i]->nlmLogIndex
+		== nlmLogEntry->nlmLogIndex) {
+		fmt_snmp_notification_log_details_varbind(s,
+							  nlmLogVarTable[i]);
+	    }
+	}
+    }
+}
+
+
+
+static int
+show_snmp_notification_log_details(scli_interp_t *interp,
+				   int argc, char **argv)
+{
+    notification_log_mib_nlmLogEntry_t **nlmLogTable;
+    notification_log_mib_nlmLogVariableEntry_t **nlmLogVarTable;
+    int i, c;
+    
+    g_return_val_if_fail(interp, SCLI_ERROR);
+
+    if (argc > 1) {
+	return SCLI_SYNTAX_NUMARGS;
+    }
+
+    if (scli_interp_dry(interp)) {
+	return SCLI_OK;
+    }
+
+    notification_log_mib_get_nlmLogTable(interp->peer, &nlmLogTable, 0);
+    if (interp->peer->error_status) {
+	return SCLI_SNMP;
+    }
+    notification_log_mib_get_nlmLogVariableTable(interp->peer,
+						 &nlmLogVarTable, 0);
+
+    if (nlmLogTable) {
+	for (i = 0, c = 0; nlmLogTable[i]; i++) {
+	    if (c) {
+		g_string_append(interp->result, "\n");
+	    }
+	    fmt_snmp_notification_log_details(interp->result, nlmLogTable[i],
+					      nlmLogVarTable);
+	    c++;
+	}
+    }
+
+    if (nlmLogTable) {
+	notification_log_mib_free_nlmLogTable(nlmLogTable);
+    }
+    if (nlmLogVarTable) {
+	notification_log_mib_free_nlmLogVariableTable(nlmLogVarTable);
+    }
+
+    return SCLI_OK;
+}
+
+
+
+static void
+fmt_snmp_notification_log_info(GString *s,
+			       snmpv2_mib_system_t *system,
+		       notification_log_mib_nlmLogEntry_t *nlmLogEntry)
+{
+    const char *e = NULL;
+    int i;
+    
+    static GNetSnmpIdentity const *notifications[]
+	= { if_mib_notifications, snmpv2_mib_notifications, NULL };
+    
+    g_string_sprintfa(s, "%7.*s ",
+		      nlmLogEntry->_nlmLogNameLength,
+		      nlmLogEntry->nlmLogName);
+    
+    g_string_sprintfa(s, "%5d ", nlmLogEntry->nlmLogIndex);
+
+    if (nlmLogEntry->nlmLogTime && system->sysUpTime) {
+	int secs = (*system->sysUpTime - *(nlmLogEntry->nlmLogTime)) / 100;
+	g_string_sprintfa(s, "%02d:%02d ", secs/60, secs % 60);
+    } else {
+	g_string_sprintfa(s, "     ");
+    }
+
+    for (i = 0; notifications[i]; i++) {
+	e = fmt_identity(notifications[i],
+			 nlmLogEntry->nlmLogNotificationID,
+			 nlmLogEntry->_nlmLogNotificationIDLength);
+	if (e) {
+	    break;
+	}
+    }
+    g_string_sprintfa(s, "%-12s ", e ? e : "");
+
+    e = fmt_taddress(nlmLogEntry->nlmLogEngineTDomain,
+		     nlmLogEntry->_nlmLogEngineTDomainLength,
+		     nlmLogEntry->nlmLogEngineTAddress,
+		     nlmLogEntry->_nlmLogEngineTAddressLength);
+    if (e) {
+	g_string_sprintfa(s, "%s ", e);
+    }
+
+    g_string_sprintfa(s, "\n");
+}
+
+
+
+static int
+show_snmp_notification_log_info(scli_interp_t *interp,
+				int argc, char **argv)
+{
+    snmpv2_mib_system_t *system = NULL;
+    notification_log_mib_nlmLogEntry_t **nlmLogTable;
+    int i;
+    
+    g_return_val_if_fail(interp, SCLI_ERROR);
+
+    if (argc > 1) {
+	return SCLI_SYNTAX_NUMARGS;
+    }
+
+    if (scli_interp_dry(interp)) {
+	return SCLI_OK;
+    }
+
+    notification_log_mib_get_nlmLogTable(interp->peer, &nlmLogTable, 0);
+    if (interp->peer->error_status) {
+	return SCLI_SNMP;
+    }
+
+    snmpv2_mib_get_system(interp->peer, &system, SNMPV2_MIB_SYSUPTIME);
+    
+    if (nlmLogTable) {
+	g_string_sprintfa(interp->header,
+			  "LOGNAME   NUM  TIME NOTIFICATION ORIGINATOR");
+	for (i = 0; nlmLogTable[i]; i++) {
+	}
+	for (i--; i >= 0; i--) {
+	    fmt_snmp_notification_log_info(interp->result, system,
+					   nlmLogTable[i]);
+	}
+    }
+
+    if (system) {
+	snmpv2_mib_free_system(system);
+    }
+
+    if (nlmLogTable) {
+	notification_log_mib_free_nlmLogTable(nlmLogTable);
+    }
+
+    return SCLI_OK;
+}
+
+
+
 static int
 set_snmp_authentication_traps(scli_interp_t *interp, int argc, char **argv)
 {
@@ -1106,8 +1518,8 @@ set_snmp_authentication_traps(scli_interp_t *interp, int argc, char **argv)
 	return SCLI_SYNTAX_NUMARGS;
     }
 
-    if (! gsnmp_enum_get_number(snmpv2_mib_enums_snmpEnableAuthenTraps,
-				argv[1], &value)) {
+    if (! gnet_snmp_enum_get_number(snmpv2_mib_enums_snmpEnableAuthenTraps,
+				    argv[1], &value)) {
 	g_string_assign(interp->result, argv[1]);
 	return SCLI_SYNTAX_VALUE;
     }
@@ -1170,8 +1582,8 @@ dump_snmp(scli_interp_t *interp, int argc, char **argv)
 	    if (! creatable(vacmGroupTable[i]->vacmSecurityToGroupStorageType)) {
 		continue;
 	    }
-	    e  = gsnmp_enum_get_label(security_model,
-				      vacmGroupTable[i]->vacmSecurityModel);
+	    e  = gnet_snmp_enum_get_label(security_model,
+					  vacmGroupTable[i]->vacmSecurityModel);
 	    g_string_sprintfa(interp->result,
 		      "create snmp vacm member \"%.*s\" \"%.*s\" \"%s\"\n",
 			      vacmGroupTable[i]->_vacmSecurityNameLength,
@@ -1342,6 +1754,41 @@ scli_init_snmp_mode(scli_interp_t *interp)
 	  NULL, NULL,
 	  show_snmp_notification_targets },
 
+	{ "show snmp contexts", NULL,
+	  "The `show snmp contexts' command displays information\n"
+	  "about the available SNMP contexts.",
+	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_DRY,
+	  NULL, NULL,
+	  show_snmp_contexts },
+
+	{ "show snmp csm", NULL,
+	  "The `show snmp csm communities' command displays information\n"
+	  "about the configured SNMP communities.",
+	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_DRY,
+	  NULL, NULL,
+	  show_snmp_csm_communities },
+
+	{ "show snmp notification log details", NULL,
+	  "The `show snmp notification log details' command displays\n"
+	  "detailed information about logged SNMP notifications.",
+	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_DRY,
+	  NULL, NULL,
+	  show_snmp_notification_log_details },
+
+	{ "show snmp notification log info", NULL,
+	  "The `show snmp notification log info' command displays\n"
+	  "summary information about logged SNMP notifications.",
+	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_DRY,
+	  NULL, NULL,
+	  show_snmp_notification_log_info },
+
+	{ "monitor snmp notification log info", NULL,
+	  "The `monitor snmp notification log info' command displays\n"
+	  "summary information about logged SNMP notifications.",
+	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_MONITOR | SCLI_CMD_FLAG_DRY,
+	  NULL, NULL,
+	  show_snmp_notification_log_info },
+
 	{ "dump snmp", NULL,
 	  "The `dump snmp' command generates a sequence of scli commands\n"
 	  "which can be used to restore the engine configuration.\n",
@@ -1356,11 +1803,12 @@ scli_init_snmp_mode(scli_interp_t *interp)
 	"snmp",
 	
 	"The snmp scli mode is based on the SNMPv2-MIB as published\n"
-	"in RFC 1907, the SNMP-FRAMEWORK-MIB as published in RFC 2571,\n"
-	"the SNMP-USER-BASED-SM-MIB as published in RFC 2574, the\n"
-	"SNMP-VIEW-BASED-ACM-MIB as published in RFC 2575, the\n"
-	"SNMP-TARGET-MIB as published in RFC 2573, and the\n"
-	"SNMP-NOTIFICATION-MIB as published in RFC 2573.",
+	"in RFC 1907, the SNMP-FRAMEWORK-MIB as published in RFC 3411,\n"
+	"the SNMP-USER-BASED-SM-MIB as published in RFC 3414, the\n"
+	"SNMP-VIEW-BASED-ACM-MIB as published in RFC 3415, the\n"
+	"SNMP-TARGET-MIB as published in RFC 3413, the\n"
+	"SNMP-NOTIFICATION-MIB as published in RFC 3413, and the"
+	"NOTIFICATION-LOG-MIB as published in RFC 3014.",
 	cmds
     };
     
