@@ -17,11 +17,12 @@
  * along with this program; if not, write to the Free Software Foundation,
  * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  * 
- * @(#) $Id: system.c,v 1.73 2002/07/19 09:17:16 schoenw Exp $
+ * @(#) $Id: system.c 740 2005-01-26 22:28:04Z schoenw $
  */
 
 #include "scli.h"
 
+#include "snmpv2-tc.h"
 #include "host-resources-mib.h"
 #include "host-resources-types.h"
 #include "snmpv2-mib.h"
@@ -29,12 +30,13 @@
 #include "entity-mib.h"
 #include "bridge-mib.h"
 #include "disman-script-mib.h"
+#include "rs-232-mib.h"
 
 #include <ctype.h>
 
 
 
-static GSnmpEnum const dot1dBaseType[] = {
+static GNetSnmpEnum const dot1dBaseType[] = {
     { BRIDGE_MIB_DOT1DBASETYPE_UNKNOWN,	"unknown" },
     { BRIDGE_MIB_DOT1DBASETYPE_TRANSPARENT_ONLY,	"transparent (TP)" },
     { BRIDGE_MIB_DOT1DBASETYPE_SOURCEROUTE_ONLY,	"source route (SR)" },
@@ -65,7 +67,7 @@ static guint32 const oid_flash_memory[]
 static guint32 const oid_network_disk[]
     = { HOST_RESOURCES_TYPES_HRSTORAGENETWORKDISK };
 
-static GSnmpIdentity const storage_types[] = {
+static GNetSnmpIdentity const storage_types[] = {
     { oid_other,
       sizeof(oid_other)/sizeof(guint32),
       "other" },
@@ -148,7 +150,7 @@ static guint32 const oid_fs_fat32[]
 static guint32 const oid_fs_ext2[]
     = { HOST_RESOURCES_TYPES_HRFSLINUXEXT2 };
 
-static GSnmpIdentity const filesystem_types[] = {
+static GNetSnmpIdentity const filesystem_types[] = {
     { oid_fs_other,
       sizeof(oid_fs_other)/sizeof(guint32),
       "other" },
@@ -222,7 +224,7 @@ static GSnmpIdentity const filesystem_types[] = {
 };
 
 
-static GSnmpEnum const hrSWRunStatus[] = {
+static GNetSnmpEnum const hrSWRunStatus[] = {
     { HOST_RESOURCES_MIB_HRSWRUNSTATUS_RUNNING,		"C" },
     { HOST_RESOURCES_MIB_HRSWRUNSTATUS_RUNNABLE,	"R" },
     { HOST_RESOURCES_MIB_HRSWRUNSTATUS_NOTRUNNABLE,	"S" },
@@ -230,7 +232,7 @@ static GSnmpEnum const hrSWRunStatus[] = {
     { 0, NULL }
 };
 
-static GSnmpEnum const hrSWRunType[] = {
+static GNetSnmpEnum const hrSWRunType[] = {
     { HOST_RESOURCES_MIB_HRSWRUNTYPE_UNKNOWN,		"?" },
     { HOST_RESOURCES_MIB_HRSWRUNTYPE_OPERATINGSYSTEM,	"O" },
     { HOST_RESOURCES_MIB_HRSWRUNTYPE_DEVICEDRIVER,	"D" },
@@ -238,7 +240,15 @@ static GSnmpEnum const hrSWRunType[] = {
     { 0, NULL }
 };
 
-GSnmpEnum const hrFSAccess[] = {
+static GNetSnmpEnum const hrSWInstalledType[] = {
+    { HOST_RESOURCES_MIB_HRSWINSTALLEDTYPE_UNKNOWN,		"?" },
+    { HOST_RESOURCES_MIB_HRSWINSTALLEDTYPE_OPERATINGSYSTEM,	"O" },
+    { HOST_RESOURCES_MIB_HRSWINSTALLEDTYPE_DEVICEDRIVER,	"D" },
+    { HOST_RESOURCES_MIB_HRSWINSTALLEDTYPE_APPLICATION,		"A" },
+    { 0, NULL }
+};
+
+static GNetSnmpEnum const hrFSAccess[] = {
     { HOST_RESOURCES_MIB_HRFSACCESS_READWRITE,	"rw" },
     { HOST_RESOURCES_MIB_HRFSACCESS_READONLY,	"ro" },
     { 0, NULL }
@@ -255,10 +265,54 @@ strip_white(guchar *s, guint16 *len)
 }
 
 
+static char*
+get_command(host_resources_mib_hrSWRunEntry_t *hrSWRunEntry)
+{
+    char *s = NULL;
+    
+    strip_white(hrSWRunEntry->hrSWRunPath,
+		&hrSWRunEntry->_hrSWRunPathLength);
+    strip_white(hrSWRunEntry->hrSWRunName,
+		&hrSWRunEntry->_hrSWRunNameLength);
+    if (hrSWRunEntry->hrSWRunPath) {
+	s = g_strdup_printf("%.*s",
+			    (int) hrSWRunEntry->_hrSWRunPathLength,
+			    hrSWRunEntry->hrSWRunPath);
+    } else if (hrSWRunEntry->hrSWRunName) {
+	s = g_strdup_printf("%.*s",
+			    (int) hrSWRunEntry->_hrSWRunNameLength,
+			    hrSWRunEntry->hrSWRunName);
+    }
+    return s;
+}
+
+
 
 static int
 match_process(regex_t *regex_path,
 	      host_resources_mib_hrSWRunEntry_t *hrSWRunEntry)
+{
+    int match = 0;
+    char *s;
+    
+    if (! regex_path) {
+	return 1;
+    }
+
+    s = get_command(hrSWRunEntry);
+    if (s) {
+	match = (regexec(regex_path, s, (size_t) 0, NULL, 0) == 0);
+	g_free(s);
+    }
+
+    return match;
+}
+
+
+
+static int
+match_software(regex_t *regex_path,
+	       host_resources_mib_hrSWInstalledEntry_t *hrSWEntry)
 {
     int status;
     
@@ -267,23 +321,14 @@ match_process(regex_t *regex_path,
     }
 
     /*
-     * Does it really make sense to filter only on the description?
-     * This way, we do not need to read the ifXTable at all...
+     * Does it really make sense to filter this way?
      */
 
-    if (hrSWRunEntry->hrSWRunPath) {
+    if (hrSWEntry->hrSWInstalledName) {
 	char *s;
-	if (hrSWRunEntry->hrSWRunParameters) {
-	    s = g_strdup_printf("%.*s %.*s",
-				(int) hrSWRunEntry->_hrSWRunPathLength,
-				hrSWRunEntry->hrSWRunPath,
-				(int) hrSWRunEntry->_hrSWRunParametersLength,
-				hrSWRunEntry->hrSWRunParameters);
-	} else {
-	    s = g_strdup_printf("%.*s",
-				(int) hrSWRunEntry->_hrSWRunPathLength,
-				hrSWRunEntry->hrSWRunPath);
-	}
+	s = g_strdup_printf("%.*s",
+			    (int) hrSWEntry->_hrSWInstalledNameLength,
+			    hrSWEntry->hrSWInstalledName);
 	status = regexec(regex_path, s, (size_t) 0, NULL, 0);
 	g_free(s);
 	if (status == 0) {
@@ -345,7 +390,7 @@ xml_system_device(xmlNodePtr root,
 		  host_resources_mib_hrDeviceEntry_t *hrDeviceEntry)
 {
     xmlNodePtr tree;
-    char const *type;
+    char const *e;
     
     tree = xmlNewChild(root, NULL, "device", NULL);
     xml_set_prop(tree, "index", "%d", hrDeviceEntry->hrDeviceIndex);
@@ -356,11 +401,11 @@ xml_system_device(xmlNodePtr root,
 				    hrDeviceEntry->hrDeviceStatus));
     }
 
-    type = gsnmp_identity_get_label(host_resources_types_identities,
-				    hrDeviceEntry->hrDeviceType,
-				    hrDeviceEntry->_hrDeviceDescrLength);
-    if (type) {
-	(void) xmlNewChild(tree, NULL, "type", type);
+    e = fmt_identity(host_resources_types_identities,
+		     hrDeviceEntry->hrDeviceType,
+		     hrDeviceEntry->_hrDeviceDescrLength);
+    if (e) {
+	(void) xmlNewChild(tree, NULL, "type", e);
     }
 	
     if (hrDeviceEntry->hrDeviceDescr && hrDeviceEntry->_hrDeviceDescrLength) {
@@ -377,19 +422,18 @@ fmt_system_device(GString *s,
 		  host_resources_mib_hrDeviceEntry_t *hrDeviceEntry,
 		  int type_width)
 {
-    char const *status;
-    char const *type;
+    char const *e;
     
     g_string_sprintfa(s, "%5u ", hrDeviceEntry->hrDeviceIndex);
 
-    status = fmt_enum(host_resources_mib_enums_hrDeviceStatus,
-		      hrDeviceEntry->hrDeviceStatus);
-    g_string_sprintfa(s, "%-8s", status ? status : "");
+    e = fmt_enum(host_resources_mib_enums_hrDeviceStatus,
+		 hrDeviceEntry->hrDeviceStatus);
+    g_string_sprintfa(s, "%-8s", e ? e : "");
 
-    type = gsnmp_identity_get_label(host_resources_types_identities,
-				    hrDeviceEntry->hrDeviceType,
-				    hrDeviceEntry->_hrDeviceDescrLength);
-    g_string_sprintfa(s, "%*s ", type_width, type ? type : "");
+    e = fmt_identity(host_resources_types_identities,
+		     hrDeviceEntry->hrDeviceType,
+		     hrDeviceEntry->_hrDeviceDescrLength);
+    g_string_sprintfa(s, "%*s ", type_width, e ? e : "");
 	
     if (hrDeviceEntry->hrDeviceDescr) {
 	g_string_sprintfa(s, "%.*s\n",
@@ -426,9 +470,9 @@ show_system_devices(scli_interp_t *interp, int argc, char **argv)
     if (hrDeviceTable) {
 	if (! scli_interp_xml(interp)) {
 	    for (i = 0; hrDeviceTable[i]; i++) {
-		type = gsnmp_identity_get_label(host_resources_types_identities,
-					hrDeviceTable[i]->hrDeviceType,
-					hrDeviceTable[i]->_hrDeviceDescrLength);
+		type = fmt_identity(host_resources_types_identities,
+				    hrDeviceTable[i]->hrDeviceType,
+				    hrDeviceTable[i]->_hrDeviceDescrLength);
 		if (type) {
 		    type_width = MAX(type_width, strlen(type));
 		}
@@ -459,6 +503,7 @@ xml_system_process(xmlNodePtr root,
 		   host_resources_mib_hrSWRunPerfEntry_t *hrSWRunPerfEntry)
 {
     xmlNodePtr tree, node;
+    char *c;
 
     tree = xmlNewChild(root, NULL, "process", NULL);
     xml_set_prop(tree, "index", "%d", hrSWRunEntry->hrSWRunIndex);
@@ -476,9 +521,9 @@ xml_system_process(xmlNodePtr root,
     }
 
     if (hrSWRunPerfEntry && hrSWRunPerfEntry->hrSWRunPerfMem) {
-	node = xml_new_child(tree, NULL, "memory", "%u",
-			     *hrSWRunPerfEntry->hrSWRunPerfMem);
-	xml_set_prop(node, "unit", "kilobyte");
+	node = xml_new_child(tree, NULL, "memory", "%llu",
+			     (guint64)1024 * *hrSWRunPerfEntry->hrSWRunPerfMem);
+	xml_set_prop(node, "unit", "bytes");
     }
     if (hrSWRunPerfEntry && hrSWRunPerfEntry->hrSWRunPerfCPU) {
 	node = xml_new_child(tree, NULL, "cpu", "%u",
@@ -486,13 +531,10 @@ xml_system_process(xmlNodePtr root,
 	xml_set_prop(node, "unit", "seconds");
     }
 
-    if (hrSWRunEntry->hrSWRunPath
-	&& hrSWRunEntry->_hrSWRunPathLength) {
-	strip_white(hrSWRunEntry->hrSWRunPath,
-		    &hrSWRunEntry->_hrSWRunPathLength);
-	(void) xml_new_child(tree, NULL, "path", "%.*s",
-			     (int) hrSWRunEntry->_hrSWRunPathLength,
-			     hrSWRunEntry->hrSWRunPath);
+    c = get_command(hrSWRunEntry);
+    if (c) {
+	(void) xml_new_child(tree, NULL, "path", "%s", c);
+	g_free(c);
     }
     if (hrSWRunEntry->hrSWRunParameters
 	&& hrSWRunEntry->_hrSWRunParametersLength) {
@@ -508,10 +550,11 @@ xml_system_process(xmlNodePtr root,
 
 static void
 fmt_system_process(GString *s,
-		    host_resources_mib_hrSWRunEntry_t *hrSWRunEntry,
-		    host_resources_mib_hrSWRunPerfEntry_t *hrSWRunPerfEntry)
+		   host_resources_mib_hrSWRunEntry_t *hrSWRunEntry,
+		   host_resources_mib_hrSWRunPerfEntry_t *hrSWRunPerfEntry)
 {
     const char *e;
+    char *c;
     
     g_string_sprintfa(s, "%7d ", hrSWRunEntry->hrSWRunIndex);
 
@@ -525,22 +568,19 @@ fmt_system_process(GString *s,
 	&& hrSWRunPerfEntry->hrSWRunPerfMem) {
 	fmt_x_kbytes(s, (guint32) *(hrSWRunPerfEntry->hrSWRunPerfMem));
     } else {
-	g_string_sprintfa(s, " %5s", "-----");
+	g_string_sprintfa(s, " -----");
     }
     if (hrSWRunPerfEntry
 	&& hrSWRunPerfEntry->hrSWRunPerfCPU) {
 	g_string_sprintfa(s, " %s",
 		  fmt_seconds((guint32) *(hrSWRunPerfEntry->hrSWRunPerfCPU)/100));
     } else {
-	g_string_sprintfa(s, " %6s", "--:--");
+	g_string_sprintfa(s, "    -:--:--");
     }
-    if (hrSWRunEntry->hrSWRunPath
-	&& hrSWRunEntry->_hrSWRunPathLength) {
-	strip_white(hrSWRunEntry->hrSWRunPath,
-		    &hrSWRunEntry->_hrSWRunPathLength);
-	g_string_sprintfa(s, " %.*s",
-			  (int) hrSWRunEntry->_hrSWRunPathLength,
-			  hrSWRunEntry->hrSWRunPath);
+    c = get_command(hrSWRunEntry);
+    if (c) {
+	g_string_sprintfa(s, " %s", c);
+	g_free(c);
     }
     if (hrSWRunEntry->hrSWRunParameters
 	&& hrSWRunEntry->_hrSWRunParametersLength) {
@@ -589,10 +629,10 @@ show_system_processes(scli_interp_t *interp, int argc, char **argv)
 
     if (hrSWRunTable) {
 	host_resources_mib_get_hrSWRunPerfTable(interp->peer,
-						&hrSWRunPerfTable, 0);
+				  &hrSWRunPerfTable, 0);
 	if (! scli_interp_xml(interp)) {
 	    g_string_append(interp->header,
-			    "    PID S T MEMORY      TIME COMMAND");
+			    "    PID S T MEMORY       TIME COMMAND");
 	}
 	for (i = 0; hrSWRunTable[i]; i++) {
 	    if (match_process(regex_path, hrSWRunTable[i])) {
@@ -622,12 +662,94 @@ show_system_processes(scli_interp_t *interp, int argc, char **argv)
 
 
 
+static void
+fmt_system_software(GString *s,
+		    host_resources_mib_hrSWInstalledEntry_t *hrSWEntry)
+{
+    const char *e;
+
+    g_string_sprintfa(s, "%7d ", hrSWEntry->hrSWInstalledIndex);
+
+    e = fmt_enum(hrSWInstalledType, hrSWEntry->hrSWInstalledType);
+    g_string_sprintfa(s, "%s ", e ? e : " ");
+
+    g_string_sprintfa(s, "%19s", fmt_date_and_time(hrSWEntry->hrSWInstalledDate,
+						   hrSWEntry->_hrSWInstalledDateLength));
+
+    if (hrSWEntry->hrSWInstalledName) {
+	strip_white(hrSWEntry->hrSWInstalledName,
+		    &hrSWEntry->_hrSWInstalledNameLength);
+        g_string_sprintfa(s, " %.*s",
+			  (int) hrSWEntry->_hrSWInstalledNameLength,
+			  hrSWEntry->hrSWInstalledName);
+    }
+    g_string_append(s, "\n");
+}
+
+
+
+static int
+show_system_software(scli_interp_t *interp, int argc, char **argv)
+{
+    host_resources_mib_hrSWInstalledEntry_t **hrSWTable = NULL;
+    regex_t _regex_path, *regex_path = NULL;
+    int i;
+
+    g_return_val_if_fail(interp, SCLI_ERROR);
+
+    if (argc > 2) {
+	return SCLI_SYNTAX_NUMARGS;
+    }
+
+    if (argc == 2) {
+	regex_path = &_regex_path;
+	if (regcomp(regex_path, argv[1], interp->regex_flags) != 0) {
+	    g_string_assign(interp->result, argv[1]);
+	    return SCLI_SYNTAX_REGEXP;
+	}
+    }
+
+    if (scli_interp_dry(interp)) {
+	if (regex_path) regfree(regex_path);
+	return SCLI_OK;
+    }
+
+    host_resources_mib_get_hrSWInstalledTable(interp->peer, &hrSWTable, 0);
+    if (interp->peer->error_status) {
+	return SCLI_SNMP;
+    }
+
+    if (hrSWTable) {
+	if (! scli_interp_xml(interp)) {
+	    g_string_append(interp->header,
+			    "    SID T DATE                SOFTWARE");
+	}
+	for (i = 0; hrSWTable[i]; i++) {
+	    if (match_software(regex_path, hrSWTable[i])) {
+		if (scli_interp_xml(interp)) {
+//		    xml_system_software(interp->xml_node, hrSWTable[i]);
+		} else {
+		    fmt_system_software(interp->result, hrSWTable[i]);
+		}
+	    }
+	}
+    }
+	
+    if (hrSWTable)
+	host_resources_mib_free_hrSWInstalledTable(hrSWTable);
+
+    if (regex_path) regfree(regex_path);
+    
+    return SCLI_OK;
+}
+
+
+
 static void
 xml_system_mount(xmlNodePtr root,
 		 host_resources_mib_hrFSEntry_t *fsEntry)
 {
     const char *e;
-    const char *type;
     xmlNodePtr tree;
     
     tree = xmlNewChild(root, NULL, "filesystem", NULL);
@@ -647,11 +769,10 @@ xml_system_mount(xmlNodePtr root,
 			     fsEntry->hrFSRemoteMountPoint);
     }
 
-    type = gsnmp_identity_get_label(filesystem_types,
-				    fsEntry->hrFSType,
-				    fsEntry->_hrFSTypeLength);
-    if (type) {
-	(void) xml_new_child(tree, NULL, "type", "%s", type);
+    e = fmt_identity(filesystem_types,
+		     fsEntry->hrFSType, fsEntry->_hrFSTypeLength);
+    if (e) {
+	(void) xml_new_child(tree, NULL, "type", "%s", e);
     }
 
     e = fmt_enum(host_resources_mib_enums_hrFSAccess,
@@ -661,7 +782,7 @@ xml_system_mount(xmlNodePtr root,
     }
 
     if (fsEntry->hrFSBootable
-	&& *(fsEntry->hrFSBootable) == HOST_RESOURCES_MIB_HRFSBOOTABLE_TRUE) {
+	&& *(fsEntry->hrFSBootable) == SNMPV2_TC_TRUTHVALUE_TRUE) {
 	(void) xmlNewChild(tree, NULL, "boot", NULL);
     }
 }
@@ -674,7 +795,6 @@ fmt_system_mount(GString *s,
 		 int loc_len, int rem_len, int type_len)
 {
     const char *e;
-    const char *type;
 
     g_string_sprintfa(s, "%5d ", fsEntry->hrFSIndex);
 
@@ -693,16 +813,16 @@ fmt_system_mount(GString *s,
 	g_string_sprintfa(s, "%*s", rem_len, "");
     }
 
-    type = gsnmp_identity_get_label(filesystem_types,
-				    fsEntry->hrFSType,
-				    fsEntry->_hrFSTypeLength);
-    g_string_sprintfa(s, "%-*s", type_len, type ? type : "");
+    e = fmt_identity(filesystem_types,
+		     fsEntry->hrFSType,
+		     fsEntry->_hrFSTypeLength);
+    g_string_sprintfa(s, "%-*s", type_len, e ? e : "");
 
     e = fmt_enum(hrFSAccess, fsEntry->hrFSAccess);
     g_string_sprintfa(s, "%s", e ? e : "");
 
     if (fsEntry->hrFSBootable
-	&& *(fsEntry->hrFSBootable) == HOST_RESOURCES_MIB_HRFSBOOTABLE_TRUE) {
+	&& *(fsEntry->hrFSBootable) == SNMPV2_TC_TRUTHVALUE_TRUE) {
 	g_string_append(s, ",boot");
     }
 
@@ -743,9 +863,9 @@ show_system_mounts(scli_interp_t *interp, int argc, char **argv)
 		rem_len = MAX(rem_len,
 			      hrFSTable[i]->_hrFSRemoteMountPointLength);
 	    }
-	    type = gsnmp_identity_get_label(filesystem_types,
-					    hrFSTable[i]->hrFSType,
-					    hrFSTable[i]->_hrFSTypeLength);
+	    type = fmt_identity(filesystem_types,
+				hrFSTable[i]->hrFSType,
+				hrFSTable[i]->_hrFSTypeLength);
 	    if (type) {
 		type_len = MAX(type_len, strlen(type));
 	    }
@@ -777,7 +897,7 @@ xml_system_storage(xmlNodePtr root,
 		   host_resources_mib_hrStorageEntry_t *hrStorageEntry)
 {
     xmlNodePtr node, tree;
-    gchar const *type = NULL;
+    gchar const *e;
 
     tree = xmlNewChild(root, NULL, "area", NULL);
     xml_set_prop(tree, "index", "%d", hrStorageEntry->hrStorageIndex);
@@ -789,11 +909,11 @@ xml_system_storage(xmlNodePtr root,
 			     hrStorageEntry->hrStorageDescr);
     }
 
-    type = gsnmp_identity_get_label(storage_types,
-				    hrStorageEntry->hrStorageType,
-				    hrStorageEntry->_hrStorageTypeLength);
-    if (type) {
-	(void) xmlNewChild(tree, NULL, "type", type);
+    e = fmt_identity(storage_types,
+		     hrStorageEntry->hrStorageType,
+		     hrStorageEntry->_hrStorageTypeLength);
+    if (e) {
+	(void) xmlNewChild(tree, NULL, "type", e);
     }
 
     if (hrStorageEntry->hrStorageAllocationUnits
@@ -812,11 +932,11 @@ xml_system_storage(xmlNodePtr root,
 	storage_used *= *(hrStorageEntry->hrStorageAllocationUnits);
 	storage_used /= scale;
 
-	node = xml_new_child(tree, NULL, "size", "%llu", storage_size);
-	xml_set_prop(node, "unit", "kilobyte");
+	node = xml_new_child(tree, NULL, "size", "%llu", (guint64)1024 * storage_size);
+	xml_set_prop(node, "unit", "bytes");
 
-	node = xml_new_child(tree, NULL, "used", "%llu", storage_used);
-	xml_set_prop(node, "unit", "kilobyte");
+	node = xml_new_child(tree, NULL, "used", "%llu", (guint64)1024 * storage_used);
+	xml_set_prop(node, "unit", "bytes");
     }
 }
 
@@ -827,7 +947,7 @@ fmt_system_storage(GString *s,
 		   host_resources_mib_hrStorageEntry_t *hrStorageEntry,
 		   int descr_width, int type_width)
 {
-    gchar const *type = NULL;
+    gchar const *e = NULL;
 
     g_string_sprintfa(s, "%5d ", hrStorageEntry->hrStorageIndex);
 
@@ -835,16 +955,16 @@ fmt_system_storage(GString *s,
 	&& hrStorageEntry->_hrStorageDescrLength) {
 	g_string_sprintfa(s, "%-*.*s",
 			  descr_width,
-			  (int) MIN(25, hrStorageEntry->_hrStorageDescrLength),
+			  (int) MIN(descr_width, hrStorageEntry->_hrStorageDescrLength),
 			  hrStorageEntry->hrStorageDescr);
     } else {
 	g_string_sprintfa(s, "%*s", descr_width, "");
     }
 
-    type = gsnmp_identity_get_label(storage_types,
-				    hrStorageEntry->hrStorageType,
-				    hrStorageEntry->_hrStorageTypeLength);
-    g_string_sprintfa(s, "%-*s", type_width, type ? type : "");
+    e = fmt_identity(storage_types,
+		     hrStorageEntry->hrStorageType,
+		     hrStorageEntry->_hrStorageTypeLength);
+    g_string_sprintfa(s, "%-*s", type_width, e ? e : "");
 
     if (hrStorageEntry->hrStorageAllocationUnits
 	&& hrStorageEntry->hrStorageSize
@@ -908,9 +1028,9 @@ show_system_storage(scli_interp_t *interp, int argc, char **argv)
 		&& hrStorageTable[i]->_hrStorageDescrLength > descr_width) {
 		descr_width = hrStorageTable[i]->_hrStorageDescrLength;
 	    }
-	    type = gsnmp_identity_get_label(storage_types,
-				    hrStorageTable[i]->hrStorageType,
-				    hrStorageTable[i]->_hrStorageTypeLength);
+	    type = fmt_identity(storage_types,
+				hrStorageTable[i]->hrStorageType,
+				hrStorageTable[i]->_hrStorageTypeLength);
 	    if (type) {
 		type_width = MAX(type_width, strlen(type));
 	    }
@@ -941,14 +1061,17 @@ static void
 xml_system_info(xmlNodePtr root, scli_interp_t *interp,
 		snmpv2_mib_system_t *system)
 {
+    GURI *uri;
+
     if (system) {
 	(void) xml_new_child(root, NULL, "name", "%.*s",
 			     (int) system->_sysNameLength,
 			     system->sysName);
-	if (interp->peer->taddress) {
-	    (void) xml_new_child(root, NULL, "taddress", "%s:%d",
-				 interp->peer->taddress,
-				 interp->peer->port);
+	uri = gnet_snmp_get_uri(interp->peer);
+	if (uri) {
+	    gchar *name = gnet_uri_get_string(uri);
+	    (void) xml_new_child(root, NULL, "uri", "%s", name);
+	    g_free(name);
 	}
 	
 	(void) xml_new_child(root, NULL, "description", "%.*s",
@@ -969,7 +1092,7 @@ xml_system_info(xmlNodePtr root, scli_interp_t *interp,
 		(void) xml_new_child(root, NULL, "vendor", "%s", vendor->name);
 	    }
 	    if (vendor && vendor->url) {
-		(void) xml_new_child(root, NULL, "url", "%s", vendor->url);
+		(void) xml_new_child(root, NULL, "vendor-url", "%s", vendor->url);
 	    }
 	}
 
@@ -996,6 +1119,7 @@ static void
 fmt_system_info(GString *s, scli_interp_t *interp,
 		snmpv2_mib_system_t *system)
 {
+    GURI *uri;
     int i;
     int const indent = 18;
 
@@ -1005,10 +1129,11 @@ fmt_system_info(GString *s, scli_interp_t *interp,
 			       (int) system->_sysNameLength,
 			       system->sysName);
 	}
-	if (interp->peer->taddress) {
-	    g_string_sprintfa(s, "%-*s", indent, "Address:");
-	    g_string_sprintfa(s, "%s:%d\n", interp->peer->taddress,
-			      interp->peer->port);
+	uri = gnet_snmp_get_uri(interp->peer);
+	if (uri) {
+	    gchar *name = gnet_uri_get_string(uri);
+	    g_string_sprintfa(s, "%-*s%s\n", indent, "Agent:", name);
+	    g_free(name);
 	}
 	if (system->sysDescr && system->_sysDescrLength) {
 	    fmt_display_string(s, indent, "Description:",
@@ -1071,11 +1196,13 @@ show_system_info(scli_interp_t *interp, int argc, char **argv)
     if_mib_ifMIBObjects_t *ifMibObjects = NULL;
     entity_mib_entityGeneral_t *entityGeneral = NULL;
     bridge_mib_dot1dBase_t *dot1dBase = NULL;
+    rs_232_mib_rs232_t *rs232 = NULL;
     disman_script_mib_smLangEntry_t **smLangTable = NULL;
     const char *e;
     GString *s;
     int i;
     int const indent = 18;
+    xmlNodePtr node;
 
     g_return_val_if_fail(interp, SCLI_ERROR);
 
@@ -1097,6 +1224,7 @@ show_system_info(scli_interp_t *interp, int argc, char **argv)
     if_mib_get_ifMIBObjects(interp->peer, &ifMibObjects, 0);
     entity_mib_get_entityGeneral(interp->peer, &entityGeneral, 0);
     bridge_mib_get_dot1dBase(interp->peer, &dot1dBase, 0);
+    rs_232_mib_get_rs232(interp->peer, &rs232, 0);
     disman_script_mib_get_smLangTable(interp->peer, &smLangTable,
 				      DISMAN_SCRIPT_MIB_SMLANGDESCR);
 
@@ -1112,10 +1240,10 @@ show_system_info(scli_interp_t *interp, int argc, char **argv)
 	    if (scli_interp_xml(interp)) {
 		(void) xml_new_child(interp->xml_node, NULL,
 				     "current-time", "%s",
-			     fmt_date_and_time(hrSystem->hrSystemDate,
+			     xml_date_and_time(hrSystem->hrSystemDate,
 					       hrSystem->_hrSystemDateLength));
 	    } else {
-		g_string_sprintfa(s, "%-*s", indent, "Current Time:");
+		g_string_sprintfa(s, "%-*s", indent, "Current-Time:");
 		g_string_append(s, fmt_date_and_time(hrSystem->hrSystemDate,
 						     hrSystem->_hrSystemDateLength));
 		g_string_append_c(s, '\n');
@@ -1128,9 +1256,9 @@ show_system_info(scli_interp_t *interp, int argc, char **argv)
 	    if (scli_interp_xml(interp)) {
 		(void) xml_new_child(interp->xml_node, NULL,
 				     "agent-boot-time", "%s",
-				     fmt_timeticks(*(system->sysUpTime)));
+				     xml_timeticks(*(system->sysUpTime)));
 	    } else {
-		g_string_sprintfa(s, "%-*s", indent, "Agent Boot Time:");
+		g_string_sprintfa(s, "%-*s", indent, "Agent-Boot-Time:");
 		g_string_append(s, fmt_timeticks(*(system->sysUpTime)));
 		g_string_append_c(s, '\n');
 	    }
@@ -1139,49 +1267,97 @@ show_system_info(scli_interp_t *interp, int argc, char **argv)
     
     if (hrSystem) {
 	if (hrSystem->hrSystemUptime) {
-	    g_string_sprintfa(s, "%-*s", indent, "System Boot Time:");
+	    if (scli_interp_xml(interp)) {
+		(void) xml_new_child(interp->xml_node, NULL,
+				     "system-boot-time", "%s",
+				     xml_timeticks(*(hrSystem->hrSystemUptime)));
+	    } else {
+	    g_string_sprintfa(s, "%-*s", indent, "System-Boot-Time:");
 	    g_string_append(s, fmt_timeticks(*(hrSystem->hrSystemUptime)));
 	    g_string_append_c(s, '\n');
 	}
+	}
 	if (hrSystem->hrSystemInitialLoadDevice) {
 	    host_resources_mib_hrDeviceEntry_t *dev;
-	    host_resources_mib_get_hrDeviceEntry(interp->peer, &dev, *hrSystem->hrSystemInitialLoadDevice, 0);
-	    if (dev->hrDeviceDescr) {
-		fmt_display_string(s, indent, "System Boot Dev:",
+	    host_resources_mib_get_hrDeviceEntry(interp->peer, &dev,
+						 *hrSystem->hrSystemInitialLoadDevice, 0);
+	    if (dev && dev->hrDeviceDescr) {
+		if (scli_interp_xml(interp)) {
+		    (void) xml_new_child(interp->xml_node, NULL,
+					 "system-boot-dev", "%.*s",
+					 (int) dev->_hrDeviceDescrLength,
+					 dev->hrDeviceDescr);
+		} else {
+		fmt_display_string(s, indent, "System-Boot-Dev:",
 				   (int) dev->_hrDeviceDescrLength,
 				   dev->hrDeviceDescr);
 	    }
+	    }
 	    host_resources_mib_free_hrDeviceEntry(dev);
 	}
 	if (hrSystem->hrSystemInitialLoadParameters) {
-	    fmt_display_string(s, indent, "System Boot Args:",
+	    if (scli_interp_xml(interp)) {
+		(void) xml_new_child(interp->xml_node, NULL,
+				     "system-boot-args", "%s",
+				     xml_display_string((int) hrSystem->_hrSystemInitialLoadParametersLength,
+							hrSystem->hrSystemInitialLoadParameters));
+	    } else {
+	    fmt_display_string(s, indent, "System-Boot-Args:",
 			       (int) hrSystem->_hrSystemInitialLoadParametersLength,
 			       hrSystem->hrSystemInitialLoadParameters);
 	}
+	}
 	if (hrSystem->hrSystemNumUsers) {
+	    if (scli_interp_xml(interp)) {
+		(void) xml_new_child(interp->xml_node, NULL,
+				     "users", "%u",
+				     *(hrSystem->hrSystemNumUsers));
+	    } else {
 	    g_string_sprintfa(s, "%-*s%u", indent, "Users:", 
 			      *(hrSystem->hrSystemNumUsers));
 	    g_string_append_c(s, '\n');
 	}
+	}
 	if (hrSystem->hrSystemProcesses) {
+	    if (scli_interp_xml(interp)) {
+		(void) xml_new_child(interp->xml_node, NULL,
+				     "processes", "%u",
+				     *(hrSystem->hrSystemProcesses));
+	    } else {
 	    g_string_sprintfa(s, "%-*s%u", indent, "Processes:",
 			      *(hrSystem->hrSystemProcesses));
+	    }
 	    if (hrSystem->hrSystemMaxProcesses
 		&& *(hrSystem->hrSystemMaxProcesses)) {
+		if (scli_interp_xml(interp)) {
+		    (void) xml_new_child(interp->xml_node, NULL,
+					 "max-processes", "%u",
+					 *(hrSystem->hrSystemMaxProcesses));
+		} else {
 		g_string_sprintfa(s, " (%u maximum)",
 				  *(hrSystem->hrSystemMaxProcesses));
 	    }
+	    }
+	    if (!scli_interp_xml(interp)) {
 	    g_string_append_c(s, '\n');
 	}
     }
+    }
 
     if (hrStorage) {
 	if (hrStorage->hrMemorySize) {
+	    if (scli_interp_xml(interp)) {
+		node = xml_new_child(interp->xml_node, NULL,
+				     "memory", "%llu",
+				     (guint64)1024 * *(hrStorage->hrMemorySize));
+		xml_set_prop(node, "unit", "bytes");
+	    } else {
 	    g_string_sprintfa(s, "%-*s", indent, "Memory:");
 	    fmt_kbytes(s, (guint32) *(hrStorage->hrMemorySize));
 	    g_string_append_c(s, '\n');
 	}
     }
+    }
 
     if (interfaces) {
 	if (interfaces->ifNumber) {
@@ -1214,21 +1390,48 @@ show_system_info(scli_interp_t *interp, int argc, char **argv)
 
     if (dot1dBase) {
 	if (dot1dBase->dot1dBaseNumPorts && *dot1dBase->dot1dBaseNumPorts) {
+	    if (scli_interp_xml(interp)) {
+		(void) xml_new_child(interp->xml_node, NULL,
+				     "bridge-ports", "%d",
+				     *(dot1dBase->dot1dBaseNumPorts));
+	    } else {
 	    g_string_sprintfa(s, "%-*s%d\n", indent, "Bridge Ports:",
 			      *(dot1dBase->dot1dBaseNumPorts));
 	}
+	}
 	if (dot1dBase->dot1dBaseType) {
-	    e = fmt_enum(dot1dBaseType, dot1dBase->dot1dBaseType);
+	    e = fmt_enum(bridge_mib_enums_dot1dBaseType, dot1dBase->dot1dBaseType);
 	    if (e) {
-		g_string_sprintfa(s, "%-*s%s\n", indent, "Bridge Type:", e);
+		if (scli_interp_xml(interp)) {
+		    (void) xml_new_child(interp->xml_node, NULL,
+					 "bridge-type", "%s", e);
+		} else {
+		    g_string_sprintfa(s, "%-*s%s\n", indent,
+				      "Bridge Type:", e);
+		}
 	    }
 	}
     }
 
+    if (rs232 && rs232->rs232Number && rs232->rs232Number > 0) {
+	if (scli_interp_xml(interp)) {
+	    (void) xml_new_child(interp->xml_node, NULL,
+				 "rs232-ports", "%d", *(rs232->rs232Number));
+	} else {
+	g_string_sprintfa(s, "%-*s%d\n", indent, "RS 232 Ports:",
+			  *(rs232->rs232Number));
+    }
+    }
+
     if (smLangTable) {
 	for (i = 0; smLangTable[i]; i++) ;
+	if (scli_interp_xml(interp)) {
+	    (void) xml_new_child(interp->xml_node, NULL,
+				 "script-languages", "%d", i);
+	} else {
 	g_string_sprintfa(s, "%-*s%u\n", indent, "Script Languages:", i);
     }
+    }
 
     if (system)
 	snmpv2_mib_free_system(system);
@@ -1244,6 +1447,8 @@ show_system_info(scli_interp_t *interp, int argc, char **argv)
 	entity_mib_free_entityGeneral(entityGeneral);
     if (dot1dBase)
 	bridge_mib_free_dot1dBase(dot1dBase);
+    if (rs232)
+	rs_232_mib_free_rs232(rs232);
     if (smLangTable)
 	disman_script_mib_free_smLangTable(smLangTable);
 
@@ -1402,6 +1607,198 @@ dump_system(scli_interp_t *interp, int argc, char **argv)
 
 
 
+static int
+check_system_contact(scli_interp_t *interp, int argc, char **argv)
+{
+    regex_t _regex_contact, *regex_contact = NULL;
+    snmpv2_mib_system_t *system;
+    int status = 1;
+
+    g_return_val_if_fail(interp, SCLI_ERROR);
+
+    if (argc > 2) {
+	return SCLI_SYNTAX_NUMARGS;
+    }
+
+    if (argc == 2) {
+	regex_contact = &_regex_contact;
+	if (regcomp(regex_contact, argv[1], interp->regex_flags) != 0) {
+	    g_string_assign(interp->result, argv[1]);
+	    return SCLI_SYNTAX_REGEXP;
+	}
+    }
+    
+    if (scli_interp_dry(interp)) {
+        if (regex_contact) regfree(regex_contact);
+	return SCLI_OK;
+    }
+
+    snmpv2_mib_get_system(interp->peer, &system, SNMPV2_MIB_SYSCONTACT);
+    if (interp->peer->error_status) {
+        if (regex_contact) regfree(regex_contact);
+	return SCLI_SNMP;
+    }
+
+    if (system) {
+	if (system->_sysContactLength == 0) {
+	    g_string_sprintfa(interp->result,
+			      "System contact is a zero-length string and "
+			      "should be configured to include at least an "
+			      "email address.\n\n");
+	}
+	if (regex_contact) {
+	    char *s = g_strdup_printf("%.*s",
+				      (int) system->_sysContactLength,
+				      system->sysContact);
+	    status = regexec(regex_contact, s, (size_t) 0, NULL, 0);
+	    g_free(s);
+	    if (status != 0) {
+		g_string_sprintfa(interp->result,
+				  "System contact `%.*s' does not match the "
+				  "regular expression `%s'.\n\n",
+				  (int) system->_sysContactLength,
+				  system->sysContact, argv[1]);
+	    }
+	}
+    }
+
+    if (system) snmpv2_mib_free_system(system);
+
+    if (regex_contact) regfree(regex_contact);
+    
+    return SCLI_OK;
+}
+
+
+
+static int
+check_system_storage(scli_interp_t *interp, int argc, char **argv)
+{
+    host_resources_mib_hrStorageEntry_t **hrStorageTable = NULL;
+    int i;
+
+    g_return_val_if_fail(interp, SCLI_ERROR);
+
+    if (argc > 1) {
+	return SCLI_SYNTAX_NUMARGS;
+    }
+    
+    if (scli_interp_dry(interp)) {
+	return SCLI_OK;
+    }
+
+    host_resources_mib_get_hrStorageTable(interp->peer,
+					  &hrStorageTable, 0);
+    if (interp->peer->error_status) {
+	return SCLI_SNMP;
+    }
+
+    if (hrStorageTable) {
+	for (i = 0; hrStorageTable[i]; i++) {
+	    if (hrStorageTable[i]->hrStorageAllocationUnits
+		&& hrStorageTable[i]->hrStorageSize
+		&& hrStorageTable[i]->hrStorageUsed) {
+		
+		guint64 storage_size = 0;
+		guint64 storage_used = 0;
+		guint32 const scale = 1024;
+		double percentage;
+		
+		storage_size = *(hrStorageTable[i]->hrStorageSize);
+		storage_size *= *(hrStorageTable[i]->hrStorageAllocationUnits);
+		storage_size /= scale;
+
+		storage_used = *(hrStorageTable[i]->hrStorageUsed);
+		storage_used *= *(hrStorageTable[i]->hrStorageAllocationUnits);
+		storage_used /= scale;
+
+		percentage = storage_size ? storage_used * 100 / storage_size : 0;
+		if (percentage > 90) {
+		    g_string_sprintfa(interp->result,
+			      "Storage #%d (%.*s) is %3.1f%% (> 90%%) filled\n\n",
+			      hrStorageTable[i]->hrStorageIndex,
+			      hrStorageTable[i]->_hrStorageDescrLength,
+			      hrStorageTable[i]->hrStorageDescr,
+				      percentage);
+		}
+	    }
+	}
+    }
+
+    if (hrStorageTable) host_resources_mib_free_hrStorageTable(hrStorageTable);
+
+    return SCLI_OK;
+}
+
+
+
+static int
+check_system_process(scli_interp_t *interp, int argc, char **argv)
+{
+    host_resources_mib_hrSWRunEntry_t **hrSWRunTable = NULL;
+    regex_t _regex_path, *regex_path = NULL;
+    int i;
+    guint min = 0, max = 0xffffffff, cnt = 0;
+
+    g_return_val_if_fail(interp, SCLI_ERROR);
+
+    if (argc > 3) {
+	return SCLI_SYNTAX_NUMARGS;
+    }
+
+    regex_path = &_regex_path;
+    if (regcomp(regex_path, (argc > 1) ? argv[1] : ".",
+		interp->regex_flags) != 0) {
+	g_string_assign(interp->result, argv[1]);
+	return SCLI_SYNTAX_REGEXP;
+    }
+
+    if (argc == 3) {
+	/* xxx scan this by hand to make sure it does what we want */
+	if (2 == sscanf(argv[2], "%u*%u", &min, &max)) {
+	} else if (1 == sscanf(argv[2], "%u", &min)) {
+	    max = min;
+	} else {
+	    if (regex_path) regfree(regex_path);
+	    return SCLI_SYNTAX_VALUE;
+	}
+    }
+
+    if (scli_interp_dry(interp)) {
+	if (regex_path) regfree(regex_path);
+	return SCLI_OK;
+    }
+
+    host_resources_mib_get_hrSWRunTable(interp->peer, &hrSWRunTable,
+					HOST_RESOURCES_MIB_HRSWRUNPATH);
+    if (interp->peer->error_status) {
+	return SCLI_SNMP;
+    }
+
+    if (hrSWRunTable) {
+	for (i = 0, cnt = 0; hrSWRunTable[i]; i++) {
+	    if (match_process(regex_path, hrSWRunTable[i])) {
+		cnt++;
+	    }
+	}
+	if (cnt < min || cnt > max) {
+	    g_string_sprintfa(interp->result,
+		  "Number of processes (%d) matching the regular "
+	          "expression `%s' does not match the bound %u*%u.\n\n",
+			      cnt, argv[1], min, max);
+	}
+    }
+	
+    if (hrSWRunTable)
+	host_resources_mib_free_hrSWRunTable(hrSWRunTable);
+
+    if (regex_path) regfree(regex_path);
+    
+    return SCLI_OK;
+}
+
+
+
 #ifdef MEM_DEBUG
 static int
 cmd_xxx(scli_interp_t *interp, int argc, char **argv)
@@ -1524,7 +1921,7 @@ scli_init_system_mode(scli_interp_t *interp)
 	  "The `show system processes' command display information about the\n"
 	  "processes currently running on the system. The regular expression\n"
 	  "<regexp> is matched against the command executed by the process\n"
-	  "to select the processes of interest.The command generates a table\n"
+	  "to select the processes of interest. The command generates a table\n"
 	  "with the following columns:\n"
 	  "\n"
 	  "  PID     process identification number\n"
@@ -1541,6 +1938,23 @@ scli_init_system_mode(scli_interp_t *interp)
 	  "system processes", NULL,
 	  show_system_processes },
 
+	{ "show system software", "[<regexp>]",
+	  "The `show system software' command display information about the\n"
+	  "software installed on the system. The regular expression <regexp>\n"
+	  "is matched against the software name to select the software of\n"
+	  "interest. The command generates a table with the following columns:\n"
+	  "\n"
+	  "  SID     software identification number\n"
+	  "  T       type of the software (see below)\n"
+	  "  DATE    software installation date\n"
+	  "  NAME    software name\n"
+	  "\n"
+	  "The software type values are ?=unknown, O=operating system, \n"
+	  "D=device driver, and A=application.",
+	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_DRY,
+	  "system software", NULL,
+	  show_system_software },
+
 	{ "monitor system storage", NULL,
 	  "The `monitor system storage' command shows the same\n"
 	  "information as the show system storage command. The\n"
@@ -1548,23 +1962,41 @@ scli_init_system_mode(scli_interp_t *interp)
 	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_MONITOR | SCLI_CMD_FLAG_DRY,
 	  NULL, NULL,
 	  show_system_storage },
-#if 0
+
 	{ "loop system storage", NULL,
-	  "The `monitor system storage' command shows the same\n"
+	  "The `loop system storage' command shows the same\n"
 	  "information as the show system storage command. The\n"
 	  "information is updated periodically.",
-	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_LOOP | SCLI_CMD_FLAG_XML,
+	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_LOOP | SCLI_CMD_FLAG_DRY,
 	  NULL, NULL,
 	  show_system_storage },
-#endif
+
 	{ "monitor system processes", "[<regexp>]",
 	  "The `monitor system processes' command show the same\n"
 	  "information as the show system processes command. The\n"
 	  "information is updated periodically.",
-	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_MONITOR,
+	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_MONITOR | SCLI_CMD_FLAG_DRY,
 	  NULL, NULL,
 	  show_system_processes },
 
+	{ "check system contact", NULL,
+	  "The `check system contact' command xxx.",
+	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_DRY,
+	  NULL, NULL,
+	  check_system_contact },
+
+	{ "check system storage", NULL,
+	  "The `check system storage' command checks xxx.",
+	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_DRY,
+	  NULL, NULL,
+	  check_system_storage },
+
+	{ "check system process", "[<regexp>] [<n>*<m>]",
+	  "The `check system process' command checks xxx.",
+	  SCLI_CMD_FLAG_NEED_PEER | SCLI_CMD_FLAG_DRY,
+	  NULL, NULL,
+	  check_system_process },
+
 #ifdef MEM_DEBUG
 	{ "xxx", "<repetitions>",
 	  "xxx",
