@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software Foundation,
  * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  * 
- * @(#) $Id: tcp.c,v 1.40 2002/07/19 09:16:26 schoenw Exp $
+ * @(#) $Id: tcp.c 733 2004-11-05 09:22:41Z schoenw $
  */
 
 #include "scli.h"
@@ -526,22 +526,22 @@ fmt_tcp_info(GString *s, tcp_mib_tcp_t *tcp)
 
     e = fmt_enum(tcp_mib_enums_tcpRtoAlgorithm, tcp->tcpRtoAlgorithm);
     if (e) {
-	g_string_sprintfa(s, "%-*s%s\n", indent, "RTO Algorithm:", e);
+	g_string_sprintfa(s, "%-*s%s\n", indent, "RTO-Algorithm:", e);
     }
     if (tcp->tcpRtoMin) {
 	g_string_sprintfa(s, "%-*s%d milliseconds\n",
-			  indent, "RTO Minimum:", *tcp->tcpRtoMin);
+			  indent, "RTO-Minimum:", *tcp->tcpRtoMin);
     }
     if (tcp->tcpRtoMax) {
 	g_string_sprintfa(s, "%-*s%d milliseconds\n",
-			  indent, "RTO Maximum:", *tcp->tcpRtoMax);
+			  indent, "RTO-Maximum:", *tcp->tcpRtoMax);
     }
     if (tcp->tcpMaxConn && *tcp->tcpMaxConn) {
 	if (*tcp->tcpMaxConn < 0) {
-	    g_string_sprintfa(s, "%-*sdynamic\n", indent, "TCB Limit:");
+	    g_string_sprintfa(s, "%-*sdynamic\n", indent, "TCB-Limit:");
 	} else {
 	    g_string_sprintfa(s, "%-*s%d connections\n",
-			      indent, "TCB Limit:", *tcp->tcpMaxConn);
+			      indent, "TCB-Limit:", *tcp->tcpMaxConn);
 	}
     }
     if (tcp->tcpCurrEstab) {
