@@ -17,12 +17,13 @@
  * along with this program; if not, write to the Free Software Foundation,
  * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  * 
- * @(#) $Id: udp.c,v 1.20 2001/12/11 02:57:20 schoenw Exp $
+ * @(#) $Id: udp.c 712 2004-10-14 20:51:26Z schoenw $
  */
 
 #include "scli.h"
 
 #include "udp-mib.h"
+
 #include "udp-mib-proc.h"
 
 
@@ -129,7 +130,7 @@ show_udp_stats(scli_interp_t *interp, int argc, char **argv)
     }
 
     if (udpStats) {
-	g_string_sprintfa(interp->result, "in %u datagrams\n", udpStats->inDatagrams);
+	g_string_sprintfa(interp->result, "in %u datagrams\n", udpStats->udpInDatagrams);
     }
 
     return SCLI_OK;
